<?php

namespace ScopedGtbabel;

/**
 * Defines allowed CSS attributes and what their values are.
 * @see HTMLPurifier_HTMLDefinition
 */
class HTMLPurifier_CSSDefinition extends \ScopedGtbabel\HTMLPurifier_Definition
{
    public $type = 'CSS';
    /**
     * Assoc array of attribute name to definition object.
     * @type HTMLPurifier_AttrDef[]
     */
    public $info = array();
    /**
     * Constructs the info array.  The meat of this class.
     * @param HTMLPurifier_Config $config
     */
    protected function doSetup($config)
    {
        $this->info['text-align'] = new \ScopedGtbabel\HTMLPurifier_AttrDef_Enum(array('left', 'right', 'center', 'justify'), \false);
        $border_style = $this->info['border-bottom-style'] = $this->info['border-right-style'] = $this->info['border-left-style'] = $this->info['border-top-style'] = new \ScopedGtbabel\HTMLPurifier_AttrDef_Enum(array('none', 'hidden', 'dotted', 'dashed', 'solid', 'double', 'groove', 'ridge', 'inset', 'outset'), \false);
        $this->info['border-style'] = new \ScopedGtbabel\HTMLPurifier_AttrDef_CSS_Multiple($border_style);
        $this->info['clear'] = new \ScopedGtbabel\HTMLPurifier_AttrDef_Enum(array('none', 'left', 'right', 'both'), \false);
        $this->info['float'] = new \ScopedGtbabel\HTMLPurifier_AttrDef_Enum(array('none', 'left', 'right'), \false);
        $this->info['font-style'] = new \ScopedGtbabel\HTMLPurifier_AttrDef_Enum(array('normal', 'italic', 'oblique'), \false);
        $this->info['font-variant'] = new \ScopedGtbabel\HTMLPurifier_AttrDef_Enum(array('normal', 'small-caps'), \false);
        $uri_or_none = new \ScopedGtbabel\HTMLPurifier_AttrDef_CSS_Composite(array(new \ScopedGtbabel\HTMLPurifier_AttrDef_Enum(array('none')), new \ScopedGtbabel\HTMLPurifier_AttrDef_CSS_URI()));
        $this->info['list-style-position'] = new \ScopedGtbabel\HTMLPurifier_AttrDef_Enum(array('inside', 'outside'), \false);
        $this->info['list-style-type'] = new \ScopedGtbabel\HTMLPurifier_AttrDef_Enum(array('disc', 'circle', 'square', 'decimal', 'lower-roman', 'upper-roman', 'lower-alpha', 'upper-alpha', 'none'), \false);
        $this->info['list-style-image'] = $uri_or_none;
        $this->info['list-style'] = new \ScopedGtbabel\HTMLPurifier_AttrDef_CSS_ListStyle($config);
        $this->info['text-transform'] = new \ScopedGtbabel\HTMLPurifier_AttrDef_Enum(array('capitalize', 'uppercase', 'lowercase', 'none'), \false);
        $this->info['color'] = new \ScopedGtbabel\HTMLPurifier_AttrDef_CSS_Color();
        $this->info['background-image'] = $uri_or_none;
        $this->info['background-repeat'] = new \ScopedGtbabel\HTMLPurifier_AttrDef_Enum(array('repeat', 'repeat-x', 'repeat-y', 'no-repeat'));
        $this->info['background-attachment'] = new \ScopedGtbabel\HTMLPurifier_AttrDef_Enum(array('scroll', 'fixed'));
        $this->info['background-position'] = new \ScopedGtbabel\HTMLPurifier_AttrDef_CSS_BackgroundPosition();
        $border_color = $this->info['border-top-color'] = $this->info['border-bottom-color'] = $this->info['border-left-color'] = $this->info['border-right-color'] = $this->info['background-color'] = new \ScopedGtbabel\HTMLPurifier_AttrDef_CSS_Composite(array(new \ScopedGtbabel\HTMLPurifier_AttrDef_Enum(array('transparent')), new \ScopedGtbabel\HTMLPurifier_AttrDef_CSS_Color()));
        $this->info['background'] = new \ScopedGtbabel\HTMLPurifier_AttrDef_CSS_Background($config);
        $this->info['border-color'] = new \ScopedGtbabel\HTMLPurifier_AttrDef_CSS_Multiple($border_color);
        $border_width = $this->info['border-top-width'] = $this->info['border-bottom-width'] = $this->info['border-left-width'] = $this->info['border-right-width'] = new \ScopedGtbabel\HTMLPurifier_AttrDef_CSS_Composite(array(new \ScopedGtbabel\HTMLPurifier_AttrDef_Enum(array('thin', 'medium', 'thick')), new \ScopedGtbabel\HTMLPurifier_AttrDef_CSS_Length('0')));
        $this->info['border-width'] = new \ScopedGtbabel\HTMLPurifier_AttrDef_CSS_Multiple($border_width);
        $this->info['letter-spacing'] = new \ScopedGtbabel\HTMLPurifier_AttrDef_CSS_Composite(array(new \ScopedGtbabel\HTMLPurifier_AttrDef_Enum(array('normal')), new \ScopedGtbabel\HTMLPurifier_AttrDef_CSS_Length()));
        $this->info['word-spacing'] = new \ScopedGtbabel\HTMLPurifier_AttrDef_CSS_Composite(array(new \ScopedGtbabel\HTMLPurifier_AttrDef_Enum(array('normal')), new \ScopedGtbabel\HTMLPurifier_AttrDef_CSS_Length()));
        $this->info['font-size'] = new \ScopedGtbabel\HTMLPurifier_AttrDef_CSS_Composite(array(new \ScopedGtbabel\HTMLPurifier_AttrDef_Enum(array('xx-small', 'x-small', 'small', 'medium', 'large', 'x-large', 'xx-large', 'larger', 'smaller')), new \ScopedGtbabel\HTMLPurifier_AttrDef_CSS_Percentage(), new \ScopedGtbabel\HTMLPurifier_AttrDef_CSS_Length()));
        $this->info['line-height'] = new \ScopedGtbabel\HTMLPurifier_AttrDef_CSS_Composite(array(
            new \ScopedGtbabel\HTMLPurifier_AttrDef_Enum(array('normal')),
            new \ScopedGtbabel\HTMLPurifier_AttrDef_CSS_Number(\true),
            // no negatives
            new \ScopedGtbabel\HTMLPurifier_AttrDef_CSS_Length('0'),
            new \ScopedGtbabel\HTMLPurifier_AttrDef_CSS_Percentage(\true),
        ));
        $margin = $this->info['margin-top'] = $this->info['margin-bottom'] = $this->info['margin-left'] = $this->info['margin-right'] = new \ScopedGtbabel\HTMLPurifier_AttrDef_CSS_Composite(array(new \ScopedGtbabel\HTMLPurifier_AttrDef_CSS_Length(), new \ScopedGtbabel\HTMLPurifier_AttrDef_CSS_Percentage(), new \ScopedGtbabel\HTMLPurifier_AttrDef_Enum(array('auto'))));
        $this->info['margin'] = new \ScopedGtbabel\HTMLPurifier_AttrDef_CSS_Multiple($margin);
        // non-negative
        $padding = $this->info['padding-top'] = $this->info['padding-bottom'] = $this->info['padding-left'] = $this->info['padding-right'] = new \ScopedGtbabel\HTMLPurifier_AttrDef_CSS_Composite(array(new \ScopedGtbabel\HTMLPurifier_AttrDef_CSS_Length('0'), new \ScopedGtbabel\HTMLPurifier_AttrDef_CSS_Percentage(\true)));
        $this->info['padding'] = new \ScopedGtbabel\HTMLPurifier_AttrDef_CSS_Multiple($padding);
        $this->info['text-indent'] = new \ScopedGtbabel\HTMLPurifier_AttrDef_CSS_Composite(array(new \ScopedGtbabel\HTMLPurifier_AttrDef_CSS_Length(), new \ScopedGtbabel\HTMLPurifier_AttrDef_CSS_Percentage()));
        $trusted_wh = new \ScopedGtbabel\HTMLPurifier_AttrDef_CSS_Composite(array(new \ScopedGtbabel\HTMLPurifier_AttrDef_CSS_Length('0'), new \ScopedGtbabel\HTMLPurifier_AttrDef_CSS_Percentage(\true), new \ScopedGtbabel\HTMLPurifier_AttrDef_Enum(array('auto', 'initial', 'inherit'))));
        $trusted_min_wh = new \ScopedGtbabel\HTMLPurifier_AttrDef_CSS_Composite(array(new \ScopedGtbabel\HTMLPurifier_AttrDef_CSS_Length('0'), new \ScopedGtbabel\HTMLPurifier_AttrDef_CSS_Percentage(\true), new \ScopedGtbabel\HTMLPurifier_AttrDef_Enum(array('initial', 'inherit'))));
        $trusted_max_wh = new \ScopedGtbabel\HTMLPurifier_AttrDef_CSS_Composite(array(new \ScopedGtbabel\HTMLPurifier_AttrDef_CSS_Length('0'), new \ScopedGtbabel\HTMLPurifier_AttrDef_CSS_Percentage(\true), new \ScopedGtbabel\HTMLPurifier_AttrDef_Enum(array('none', 'initial', 'inherit'))));
        $max = $config->get('CSS.MaxImgLength');
        $this->info['width'] = $this->info['height'] = $max === null ? $trusted_wh : new \ScopedGtbabel\HTMLPurifier_AttrDef_Switch(
            'img',
            // For img tags:
            new \ScopedGtbabel\HTMLPurifier_AttrDef_CSS_Composite(array(new \ScopedGtbabel\HTMLPurifier_AttrDef_CSS_Length('0', $max), new \ScopedGtbabel\HTMLPurifier_AttrDef_Enum(array('auto')))),
            // For everyone else:
            $trusted_wh
        );
        $this->info['min-width'] = $this->info['min-height'] = $max === null ? $trusted_min_wh : new \ScopedGtbabel\HTMLPurifier_AttrDef_Switch(
            'img',
            // For img tags:
            new \ScopedGtbabel\HTMLPurifier_AttrDef_CSS_Composite(array(new \ScopedGtbabel\HTMLPurifier_AttrDef_CSS_Length('0', $max), new \ScopedGtbabel\HTMLPurifier_AttrDef_Enum(array('initial', 'inherit')))),
            // For everyone else:
            $trusted_min_wh
        );
        $this->info['max-width'] = $this->info['max-height'] = $max === null ? $trusted_max_wh : new \ScopedGtbabel\HTMLPurifier_AttrDef_Switch(
            'img',
            // For img tags:
            new \ScopedGtbabel\HTMLPurifier_AttrDef_CSS_Composite(array(new \ScopedGtbabel\HTMLPurifier_AttrDef_CSS_Length('0', $max), new \ScopedGtbabel\HTMLPurifier_AttrDef_Enum(array('none', 'initial', 'inherit')))),
            // For everyone else:
            $trusted_max_wh
        );
        $this->info['text-decoration'] = new \ScopedGtbabel\HTMLPurifier_AttrDef_CSS_TextDecoration();
        $this->info['font-family'] = new \ScopedGtbabel\HTMLPurifier_AttrDef_CSS_FontFamily();
        // this could use specialized code
        $this->info['font-weight'] = new \ScopedGtbabel\HTMLPurifier_AttrDef_Enum(array('normal', 'bold', 'bolder', 'lighter', '100', '200', '300', '400', '500', '600', '700', '800', '900'), \false);
        // MUST be called after other font properties, as it references
        // a CSSDefinition object
        $this->info['font'] = new \ScopedGtbabel\HTMLPurifier_AttrDef_CSS_Font($config);
        // same here
        $this->info['border'] = $this->info['border-bottom'] = $this->info['border-top'] = $this->info['border-left'] = $this->info['border-right'] = new \ScopedGtbabel\HTMLPurifier_AttrDef_CSS_Border($config);
        $this->info['border-collapse'] = new \ScopedGtbabel\HTMLPurifier_AttrDef_Enum(array('collapse', 'separate'));
        $this->info['caption-side'] = new \ScopedGtbabel\HTMLPurifier_AttrDef_Enum(array('top', 'bottom'));
        $this->info['table-layout'] = new \ScopedGtbabel\HTMLPurifier_AttrDef_Enum(array('auto', 'fixed'));
        $this->info['vertical-align'] = new \ScopedGtbabel\HTMLPurifier_AttrDef_CSS_Composite(array(new \ScopedGtbabel\HTMLPurifier_AttrDef_Enum(array('baseline', 'sub', 'super', 'top', 'text-top', 'middle', 'bottom', 'text-bottom')), new \ScopedGtbabel\HTMLPurifier_AttrDef_CSS_Length(), new \ScopedGtbabel\HTMLPurifier_AttrDef_CSS_Percentage()));
        $this->info['border-spacing'] = new \ScopedGtbabel\HTMLPurifier_AttrDef_CSS_Multiple(new \ScopedGtbabel\HTMLPurifier_AttrDef_CSS_Length(), 2);
        // These CSS properties don't work on many browsers, but we live
        // in THE FUTURE!
        $this->info['white-space'] = new \ScopedGtbabel\HTMLPurifier_AttrDef_Enum(array('nowrap', 'normal', 'pre', 'pre-wrap', 'pre-line'));
        if ($config->get('CSS.Proprietary')) {
            $this->doSetupProprietary($config);
        }
        if ($config->get('CSS.AllowTricky')) {
            $this->doSetupTricky($config);
        }
        if ($config->get('CSS.Trusted')) {
            $this->doSetupTrusted($config);
        }
        $allow_important = $config->get('CSS.AllowImportant');
        // wrap all attr-defs with decorator that handles !important
        foreach ($this->info as $k => $v) {
            $this->info[$k] = new \ScopedGtbabel\HTMLPurifier_AttrDef_CSS_ImportantDecorator($v, $allow_important);
        }
        $this->setupConfigStuff($config);
    }
    /**
     * @param HTMLPurifier_Config $config
     */
    protected function doSetupProprietary($config)
    {
        // Internet Explorer only scrollbar colors
        $this->info['scrollbar-arrow-color'] = new \ScopedGtbabel\HTMLPurifier_AttrDef_CSS_Color();
        $this->info['scrollbar-base-color'] = new \ScopedGtbabel\HTMLPurifier_AttrDef_CSS_Color();
        $this->info['scrollbar-darkshadow-color'] = new \ScopedGtbabel\HTMLPurifier_AttrDef_CSS_Color();
        $this->info['scrollbar-face-color'] = new \ScopedGtbabel\HTMLPurifier_AttrDef_CSS_Color();
        $this->info['scrollbar-highlight-color'] = new \ScopedGtbabel\HTMLPurifier_AttrDef_CSS_Color();
        $this->info['scrollbar-shadow-color'] = new \ScopedGtbabel\HTMLPurifier_AttrDef_CSS_Color();
        // vendor specific prefixes of opacity
        $this->info['-moz-opacity'] = new \ScopedGtbabel\HTMLPurifier_AttrDef_CSS_AlphaValue();
        $this->info['-khtml-opacity'] = new \ScopedGtbabel\HTMLPurifier_AttrDef_CSS_AlphaValue();
        // only opacity, for now
        $this->info['filter'] = new \ScopedGtbabel\HTMLPurifier_AttrDef_CSS_Filter();
        // more CSS3
        $this->info['page-break-after'] = $this->info['page-break-before'] = new \ScopedGtbabel\HTMLPurifier_AttrDef_Enum(array('auto', 'always', 'avoid', 'left', 'right'));
        $this->info['page-break-inside'] = new \ScopedGtbabel\HTMLPurifier_AttrDef_Enum(array('auto', 'avoid'));
        $border_radius = new \ScopedGtbabel\HTMLPurifier_AttrDef_CSS_Composite(array(
            new \ScopedGtbabel\HTMLPurifier_AttrDef_CSS_Percentage(\true),
            // disallow negative
            new \ScopedGtbabel\HTMLPurifier_AttrDef_CSS_Length('0'),
        ));
        $this->info['border-top-left-radius'] = $this->info['border-top-right-radius'] = $this->info['border-bottom-right-radius'] = $this->info['border-bottom-left-radius'] = new \ScopedGtbabel\HTMLPurifier_AttrDef_CSS_Multiple($border_radius, 2);
        // TODO: support SLASH syntax
        $this->info['border-radius'] = new \ScopedGtbabel\HTMLPurifier_AttrDef_CSS_Multiple($border_radius, 4);
    }
    /**
     * @param HTMLPurifier_Config $config
     */
    protected function doSetupTricky($config)
    {
        $this->info['display'] = new \ScopedGtbabel\HTMLPurifier_AttrDef_Enum(array('inline', 'block', 'list-item', 'run-in', 'compact', 'marker', 'table', 'inline-block', 'inline-table', 'table-row-group', 'table-header-group', 'table-footer-group', 'table-row', 'table-column-group', 'table-column', 'table-cell', 'table-caption', 'none'));
        $this->info['visibility'] = new \ScopedGtbabel\HTMLPurifier_AttrDef_Enum(array('visible', 'hidden', 'collapse'));
        $this->info['overflow'] = new \ScopedGtbabel\HTMLPurifier_AttrDef_Enum(array('visible', 'hidden', 'auto', 'scroll'));
        $this->info['opacity'] = new \ScopedGtbabel\HTMLPurifier_AttrDef_CSS_AlphaValue();
    }
    /**
     * @param HTMLPurifier_Config $config
     */
    protected function doSetupTrusted($config)
    {
        $this->info['position'] = new \ScopedGtbabel\HTMLPurifier_AttrDef_Enum(array('static', 'relative', 'absolute', 'fixed'));
        $this->info['top'] = $this->info['left'] = $this->info['right'] = $this->info['bottom'] = new \ScopedGtbabel\HTMLPurifier_AttrDef_CSS_Composite(array(new \ScopedGtbabel\HTMLPurifier_AttrDef_CSS_Length(), new \ScopedGtbabel\HTMLPurifier_AttrDef_CSS_Percentage(), new \ScopedGtbabel\HTMLPurifier_AttrDef_Enum(array('auto'))));
        $this->info['z-index'] = new \ScopedGtbabel\HTMLPurifier_AttrDef_CSS_Composite(array(new \ScopedGtbabel\HTMLPurifier_AttrDef_Integer(), new \ScopedGtbabel\HTMLPurifier_AttrDef_Enum(array('auto'))));
    }
    /**
     * Performs extra config-based processing. Based off of
     * HTMLPurifier_HTMLDefinition.
     * @param HTMLPurifier_Config $config
     * @todo Refactor duplicate elements into common class (probably using
     *       composition, not inheritance).
     */
    protected function setupConfigStuff($config)
    {
        // setup allowed elements
        $support = "(for information on implementing this, see the " . "support forums) ";
        $allowed_properties = $config->get('CSS.AllowedProperties');
        if ($allowed_properties !== null) {
            foreach ($this->info as $name => $d) {
                if (!isset($allowed_properties[$name])) {
                    unset($this->info[$name]);
                }
                unset($allowed_properties[$name]);
            }
            // emit errors
            foreach ($allowed_properties as $name => $d) {
                // :TODO: Is this htmlspecialchars() call really necessary?
                $name = \htmlspecialchars($name);
                \trigger_error("Style attribute '{$name}' is not supported {$support}", \E_USER_WARNING);
            }
        }
        $forbidden_properties = $config->get('CSS.ForbiddenProperties');
        if ($forbidden_properties !== null) {
            foreach ($this->info as $name => $d) {
                if (isset($forbidden_properties[$name])) {
                    unset($this->info[$name]);
                }
            }
        }
    }
}
// vim: et sw=4 sts=4
