<?php

namespace ScopedGtbabel\Matrix;

use ScopedGtbabel\Matrix\Operators\Addition;
use ScopedGtbabel\Matrix\Operators\DirectSum;
use ScopedGtbabel\Matrix\Operators\Division;
use ScopedGtbabel\Matrix\Operators\Multiplication;
use ScopedGtbabel\Matrix\Operators\Subtraction;
class Operations
{
    public static function add(...$matrixValues) : \ScopedGtbabel\Matrix\Matrix
    {
        if (\count($matrixValues) < 2) {
            throw new \ScopedGtbabel\Matrix\Exception('Addition operation requires at least 2 arguments');
        }
        $matrix = \array_shift($matrixValues);
        if (\is_array($matrix)) {
            $matrix = new \ScopedGtbabel\Matrix\Matrix($matrix);
        }
        if (!$matrix instanceof \ScopedGtbabel\Matrix\Matrix) {
            throw new \ScopedGtbabel\Matrix\Exception('Addition arguments must be Matrix or array');
        }
        $result = new \ScopedGtbabel\Matrix\Operators\Addition($matrix);
        foreach ($matrixValues as $matrix) {
            $result->execute($matrix);
        }
        return $result->result();
    }
    public static function directsum(...$matrixValues) : \ScopedGtbabel\Matrix\Matrix
    {
        if (\count($matrixValues) < 2) {
            throw new \ScopedGtbabel\Matrix\Exception('DirectSum operation requires at least 2 arguments');
        }
        $matrix = \array_shift($matrixValues);
        if (\is_array($matrix)) {
            $matrix = new \ScopedGtbabel\Matrix\Matrix($matrix);
        }
        if (!$matrix instanceof \ScopedGtbabel\Matrix\Matrix) {
            throw new \ScopedGtbabel\Matrix\Exception('DirectSum arguments must be Matrix or array');
        }
        $result = new \ScopedGtbabel\Matrix\Operators\DirectSum($matrix);
        foreach ($matrixValues as $matrix) {
            $result->execute($matrix);
        }
        return $result->result();
    }
    public static function divideby(...$matrixValues) : \ScopedGtbabel\Matrix\Matrix
    {
        if (\count($matrixValues) < 2) {
            throw new \ScopedGtbabel\Matrix\Exception('Division operation requires at least 2 arguments');
        }
        $matrix = \array_shift($matrixValues);
        if (\is_array($matrix)) {
            $matrix = new \ScopedGtbabel\Matrix\Matrix($matrix);
        }
        if (!$matrix instanceof \ScopedGtbabel\Matrix\Matrix) {
            throw new \ScopedGtbabel\Matrix\Exception('Division arguments must be Matrix or array');
        }
        $result = new \ScopedGtbabel\Matrix\Operators\Division($matrix);
        foreach ($matrixValues as $matrix) {
            $result->execute($matrix);
        }
        return $result->result();
    }
    public static function divideinto(...$matrixValues) : \ScopedGtbabel\Matrix\Matrix
    {
        if (\count($matrixValues) < 2) {
            throw new \ScopedGtbabel\Matrix\Exception('Division operation requires at least 2 arguments');
        }
        $matrix = \array_pop($matrixValues);
        $matrixValues = \array_reverse($matrixValues);
        if (\is_array($matrix)) {
            $matrix = new \ScopedGtbabel\Matrix\Matrix($matrix);
        }
        if (!$matrix instanceof \ScopedGtbabel\Matrix\Matrix) {
            throw new \ScopedGtbabel\Matrix\Exception('Division arguments must be Matrix or array');
        }
        $result = new \ScopedGtbabel\Matrix\Operators\Division($matrix);
        foreach ($matrixValues as $matrix) {
            $result->execute($matrix);
        }
        return $result->result();
    }
    public static function multiply(...$matrixValues) : \ScopedGtbabel\Matrix\Matrix
    {
        if (\count($matrixValues) < 2) {
            throw new \ScopedGtbabel\Matrix\Exception('Multiplication operation requires at least 2 arguments');
        }
        $matrix = \array_shift($matrixValues);
        if (\is_array($matrix)) {
            $matrix = new \ScopedGtbabel\Matrix\Matrix($matrix);
        }
        if (!$matrix instanceof \ScopedGtbabel\Matrix\Matrix) {
            throw new \ScopedGtbabel\Matrix\Exception('Multiplication arguments must be Matrix or array');
        }
        $result = new \ScopedGtbabel\Matrix\Operators\Multiplication($matrix);
        foreach ($matrixValues as $matrix) {
            $result->execute($matrix);
        }
        return $result->result();
    }
    public static function subtract(...$matrixValues) : \ScopedGtbabel\Matrix\Matrix
    {
        if (\count($matrixValues) < 2) {
            throw new \ScopedGtbabel\Matrix\Exception('Subtraction operation requires at least 2 arguments');
        }
        $matrix = \array_shift($matrixValues);
        if (\is_array($matrix)) {
            $matrix = new \ScopedGtbabel\Matrix\Matrix($matrix);
        }
        if (!$matrix instanceof \ScopedGtbabel\Matrix\Matrix) {
            throw new \ScopedGtbabel\Matrix\Exception('Subtraction arguments must be Matrix or array');
        }
        $result = new \ScopedGtbabel\Matrix\Operators\Subtraction($matrix);
        foreach ($matrixValues as $matrix) {
            $result->execute($matrix);
        }
        return $result->result();
    }
}
