<?php

namespace ScopedGtbabel\Matrix\Operators;

use ScopedGtbabel\Matrix\Div0Exception;
use ScopedGtbabel\Matrix\Exception;
use ScopedGtbabel\Matrix\Matrix;
use ScopedGtbabel\Matrix\Functions;
class Division extends \ScopedGtbabel\Matrix\Operators\Multiplication
{
    /**
     * Execute the division
     *
     * @param mixed $value The matrix or numeric value to divide the current base value by
     * @throws Exception If the provided argument is not appropriate for the operation
     * @return $this The operation object, allowing multiple divisions to be chained
     **/
    public function execute($value, string $type = 'division') : \ScopedGtbabel\Matrix\Operators\Operator
    {
        if (\is_array($value)) {
            $value = new \ScopedGtbabel\Matrix\Matrix($value);
        }
        if (\is_object($value) && $value instanceof \ScopedGtbabel\Matrix\Matrix) {
            $value = \ScopedGtbabel\Matrix\Functions::inverse($value, $type);
            return $this->multiplyMatrix($value, $type);
        } elseif (\is_numeric($value)) {
            return $this->multiplyScalar(1 / $value, $type);
        }
        throw new \ScopedGtbabel\Matrix\Exception('Invalid argument for division');
    }
}
