<?php

namespace ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\DateTimeExcel;

use DateTimeInterface;
use ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Exception;
use ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Functions;
use ScopedGtbabel\PhpOffice\PhpSpreadsheet\Shared\Date as SharedDateHelper;
class Days
{
    /**
     * DAYS.
     *
     * Returns the number of days between two dates
     *
     * Excel Function:
     *        DAYS(endDate, startDate)
     *
     * @param DateTimeInterface|float|int|string $endDate Excel date serial value (float),
     * PHP date timestamp (integer), PHP DateTime object, or a standard date string
     * @param DateTimeInterface|float|int|string $startDate Excel date serial value (float),
     * PHP date timestamp (integer), PHP DateTime object, or a standard date string
     *
     * @return int|string Number of days between start date and end date or an error
     */
    public static function between($endDate, $startDate)
    {
        try {
            $startDate = \ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\DateTimeExcel\Helpers::getDateValue($startDate);
            $endDate = \ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\DateTimeExcel\Helpers::getDateValue($endDate);
        } catch (\ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Exception $e) {
            return $e->getMessage();
        }
        // Execute function
        $PHPStartDateObject = \ScopedGtbabel\PhpOffice\PhpSpreadsheet\Shared\Date::excelToDateTimeObject($startDate);
        $PHPEndDateObject = \ScopedGtbabel\PhpOffice\PhpSpreadsheet\Shared\Date::excelToDateTimeObject($endDate);
        $days = \ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Functions::VALUE();
        $diff = $PHPStartDateObject->diff($PHPEndDateObject);
        if ($diff !== \false && !\is_bool($diff->days)) {
            $days = $diff->days;
            if ($diff->invert) {
                $days = -$days;
            }
        }
        return $days;
    }
}
