<?php

namespace ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\DateTimeExcel;

use ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Exception;
use ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Functions;
use ScopedGtbabel\PhpOffice\PhpSpreadsheet\Shared\Date as SharedDateHelper;
class TimeParts
{
    /**
     * HOUROFDAY.
     *
     * Returns the hour of a time value.
     * The hour is given as an integer, ranging from 0 (12:00 A.M.) to 23 (11:00 P.M.).
     *
     * Excel Function:
     *        HOUR(timeValue)
     *
     * @param mixed $timeValue Excel date serial value (float), PHP date timestamp (integer),
     *                                    PHP DateTime object, or a standard time string
     *
     * @return int|string Hour
     */
    public static function hour($timeValue)
    {
        try {
            $timeValue = \ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Functions::flattenSingleValue($timeValue);
            \ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\DateTimeExcel\Helpers::nullFalseTrueToNumber($timeValue);
            if (!\is_numeric($timeValue)) {
                $timeValue = \ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\DateTimeExcel\Helpers::getTimeValue($timeValue);
            }
            \ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\DateTimeExcel\Helpers::validateNotNegative($timeValue);
        } catch (\ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Exception $e) {
            return $e->getMessage();
        }
        // Execute function
        $timeValue = \fmod($timeValue, 1);
        $timeValue = \ScopedGtbabel\PhpOffice\PhpSpreadsheet\Shared\Date::excelToDateTimeObject($timeValue);
        return (int) $timeValue->format('H');
    }
    /**
     * MINUTE.
     *
     * Returns the minutes of a time value.
     * The minute is given as an integer, ranging from 0 to 59.
     *
     * Excel Function:
     *        MINUTE(timeValue)
     *
     * @param mixed $timeValue Excel date serial value (float), PHP date timestamp (integer),
     *                                    PHP DateTime object, or a standard time string
     *
     * @return int|string Minute
     */
    public static function minute($timeValue)
    {
        try {
            $timeValue = \ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Functions::flattenSingleValue($timeValue);
            \ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\DateTimeExcel\Helpers::nullFalseTrueToNumber($timeValue);
            if (!\is_numeric($timeValue)) {
                $timeValue = \ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\DateTimeExcel\Helpers::getTimeValue($timeValue);
            }
            \ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\DateTimeExcel\Helpers::validateNotNegative($timeValue);
        } catch (\ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Exception $e) {
            return $e->getMessage();
        }
        // Execute function
        $timeValue = \fmod($timeValue, 1);
        $timeValue = \ScopedGtbabel\PhpOffice\PhpSpreadsheet\Shared\Date::excelToDateTimeObject($timeValue);
        return (int) $timeValue->format('i');
    }
    /**
     * SECOND.
     *
     * Returns the seconds of a time value.
     * The minute is given as an integer, ranging from 0 to 59.
     *
     * Excel Function:
     *        SECOND(timeValue)
     *
     * @param mixed $timeValue Excel date serial value (float), PHP date timestamp (integer),
     *                                    PHP DateTime object, or a standard time string
     *
     * @return int|string Second
     */
    public static function second($timeValue)
    {
        try {
            $timeValue = \ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Functions::flattenSingleValue($timeValue);
            \ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\DateTimeExcel\Helpers::nullFalseTrueToNumber($timeValue);
            if (!\is_numeric($timeValue)) {
                $timeValue = \ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\DateTimeExcel\Helpers::getTimeValue($timeValue);
            }
            \ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\DateTimeExcel\Helpers::validateNotNegative($timeValue);
        } catch (\ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Exception $e) {
            return $e->getMessage();
        }
        // Execute function
        $timeValue = \fmod($timeValue, 1);
        $timeValue = \ScopedGtbabel\PhpOffice\PhpSpreadsheet\Shared\Date::excelToDateTimeObject($timeValue);
        return (int) $timeValue->format('s');
    }
}
