<?php

namespace ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\LookupRef;

use ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Functions;
use ScopedGtbabel\PhpOffice\PhpSpreadsheet\Cell\Cell;
class Hyperlink
{
    /**
     * HYPERLINK.
     *
     * Excel Function:
     *        =HYPERLINK(linkURL, [displayName])
     *
     * @param mixed $linkURL Expect string. Value to check, is also the value returned when no error
     * @param mixed $displayName Expect string. Value to return when testValue is an error condition
     * @param Cell $pCell The cell to set the hyperlink in
     *
     * @return mixed The value of $displayName (or $linkURL if $displayName was blank)
     */
    public static function set($linkURL = '', $displayName = null, ?\ScopedGtbabel\PhpOffice\PhpSpreadsheet\Cell\Cell $pCell = null)
    {
        $linkURL = $linkURL === null ? '' : \ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Functions::flattenSingleValue($linkURL);
        $displayName = $displayName === null ? '' : \ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Functions::flattenSingleValue($displayName);
        if (!\is_object($pCell) || \trim($linkURL) == '') {
            return \ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Functions::REF();
        }
        if (\is_object($displayName) || \trim($displayName) == '') {
            $displayName = $linkURL;
        }
        $pCell->getHyperlink()->setUrl($linkURL);
        $pCell->getHyperlink()->setTooltip($displayName);
        return $displayName;
    }
}
