<?php

namespace ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\MathTrig;

use ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Exception;
class Combinations
{
    /**
     * COMBIN.
     *
     * Returns the number of combinations for a given number of items. Use COMBIN to
     *        determine the total possible number of groups for a given number of items.
     *
     * Excel Function:
     *        COMBIN(numObjs,numInSet)
     *
     * @param mixed $numObjs Number of different objects
     * @param mixed $numInSet Number of objects in each combination
     *
     * @return float|int|string Number of combinations, or a string containing an error
     */
    public static function withoutRepetition($numObjs, $numInSet)
    {
        try {
            $numObjs = \ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\MathTrig\Helpers::validateNumericNullSubstitution($numObjs, null);
            $numInSet = \ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\MathTrig\Helpers::validateNumericNullSubstitution($numInSet, null);
            \ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\MathTrig\Helpers::validateNotNegative($numInSet);
            \ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\MathTrig\Helpers::validateNotNegative($numObjs - $numInSet);
        } catch (\ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Exception $e) {
            return $e->getMessage();
        }
        return \round(\ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\MathTrig\Factorial::fact($numObjs) / \ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\MathTrig\Factorial::fact($numObjs - $numInSet)) / \ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\MathTrig\Factorial::fact($numInSet);
    }
    /**
     * COMBIN.
     *
     * Returns the number of combinations for a given number of items. Use COMBIN to
     *        determine the total possible number of groups for a given number of items.
     *
     * Excel Function:
     *        COMBIN(numObjs,numInSet)
     *
     * @param mixed $numObjs Number of different objects
     * @param mixed $numInSet Number of objects in each combination
     *
     * @return float|int|string Number of combinations, or a string containing an error
     */
    public static function withRepetition($numObjs, $numInSet)
    {
        try {
            $numObjs = \ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\MathTrig\Helpers::validateNumericNullSubstitution($numObjs, null);
            $numInSet = \ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\MathTrig\Helpers::validateNumericNullSubstitution($numInSet, null);
            \ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\MathTrig\Helpers::validateNotNegative($numInSet);
            \ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\MathTrig\Helpers::validateNotNegative($numObjs);
            $numObjs = (int) $numObjs;
            $numInSet = (int) $numInSet;
            // Microsoft documentation says following is true, but Excel
            //  does not enforce this restriction.
            //Helpers::validateNotNegative($numObjs - $numInSet);
            if ($numObjs === 0) {
                \ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\MathTrig\Helpers::validateNotNegative(-$numInSet);
                return 1;
            }
        } catch (\ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Exception $e) {
            return $e->getMessage();
        }
        return \round(\ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\MathTrig\Factorial::fact($numObjs + $numInSet - 1) / \ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\MathTrig\Factorial::fact($numObjs - 1)) / \ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\MathTrig\Factorial::fact($numInSet);
    }
}
