<?php

namespace ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Statistical;

use ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Exception;
use ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Functions;
class Confidence
{
    /**
     * CONFIDENCE.
     *
     * Returns the confidence interval for a population mean
     *
     * @param mixed $alpha As a float
     * @param mixed $stdDev Standard Deviation as a float
     * @param mixed $size As an integer
     *
     * @return float|string
     */
    public static function CONFIDENCE($alpha, $stdDev, $size)
    {
        $alpha = \ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Functions::flattenSingleValue($alpha);
        $stdDev = \ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Functions::flattenSingleValue($stdDev);
        $size = \ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Functions::flattenSingleValue($size);
        try {
            $alpha = \ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Statistical\StatisticalValidations::validateFloat($alpha);
            $stdDev = \ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Statistical\StatisticalValidations::validateFloat($stdDev);
            $size = \ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Statistical\StatisticalValidations::validateInt($size);
        } catch (\ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Exception $e) {
            return $e->getMessage();
        }
        if ($alpha <= 0 || $alpha >= 1 || $stdDev <= 0 || $size < 1) {
            return \ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Functions::NAN();
        }
        return \ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Statistical\Distributions\StandardNormal::inverse(1 - $alpha / 2) * $stdDev / \sqrt($size);
    }
}
