<?php

namespace ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Statistical\Distributions;

use ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Exception;
use ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Functions;
class Fisher
{
    /**
     * FISHER.
     *
     * Returns the Fisher transformation at x. This transformation produces a function that
     *        is normally distributed rather than skewed. Use this function to perform hypothesis
     *        testing on the correlation coefficient.
     *
     * @param mixed $value Float value for which we want the probability
     *
     * @return float|string
     */
    public static function distribution($value)
    {
        $value = \ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Functions::flattenSingleValue($value);
        try {
            \ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Statistical\Distributions\DistributionValidations::validateFloat($value);
        } catch (\ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Exception $e) {
            return $e->getMessage();
        }
        if ($value <= -1 || $value >= 1) {
            return \ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Functions::NAN();
        }
        return 0.5 * \log((1 + $value) / (1 - $value));
    }
    /**
     * FISHERINV.
     *
     * Returns the inverse of the Fisher transformation. Use this transformation when
     *        analyzing correlations between ranges or arrays of data. If y = FISHER(x), then
     *        FISHERINV(y) = x.
     *
     * @param mixed $probability Float probability at which you want to evaluate the distribution
     *
     * @return float|string
     */
    public static function inverse($probability)
    {
        $probability = \ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Functions::flattenSingleValue($probability);
        try {
            \ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Statistical\Distributions\DistributionValidations::validateFloat($probability);
        } catch (\ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Exception $e) {
            return $e->getMessage();
        }
        return (\exp(2 * $probability) - 1) / (\exp(2 * $probability) + 1);
    }
}
