<?php

namespace ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Statistical\Distributions;

use ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Exception;
use ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Functions;
use ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\MathTrig;
class Poisson
{
    /**
     * POISSON.
     *
     * Returns the Poisson distribution. A common application of the Poisson distribution
     * is predicting the number of events over a specific time, such as the number of
     * cars arriving at a toll plaza in 1 minute.
     *
     * @param mixed $value Float value for which we want the probability
     * @param mixed $mean Mean value as a float
     * @param mixed $cumulative Boolean value indicating if we want the cdf (true) or the pdf (false)
     *
     * @return float|string The result, or a string containing an error
     */
    public static function distribution($value, $mean, $cumulative)
    {
        $value = \ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Functions::flattenSingleValue($value);
        $mean = \ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Functions::flattenSingleValue($mean);
        try {
            $value = \ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Statistical\Distributions\DistributionValidations::validateFloat($value);
            $mean = \ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Statistical\Distributions\DistributionValidations::validateFloat($mean);
            $cumulative = \ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Statistical\Distributions\DistributionValidations::validateBool($cumulative);
        } catch (\ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Exception $e) {
            return $e->getMessage();
        }
        if ($value < 0 || $mean < 0) {
            return \ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Functions::NAN();
        }
        if ($cumulative) {
            $summer = 0;
            $floor = \floor($value);
            for ($i = 0; $i <= $floor; ++$i) {
                $summer += $mean ** $i / \ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\MathTrig\Factorial::fact($i);
            }
            return \exp(0 - $mean) * $summer;
        }
        return \exp(0 - $mean) * $mean ** $value / \ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\MathTrig\Factorial::fact($value);
    }
}
