<?php

namespace ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Statistical;

use ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Exception;
use ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Functions;
class Standardize extends \ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Statistical\StatisticalValidations
{
    /**
     * STANDARDIZE.
     *
     * Returns a normalized value from a distribution characterized by mean and standard_dev.
     *
     * @param float $value Value to normalize
     * @param float $mean Mean Value
     * @param float $stdDev Standard Deviation
     *
     * @return float|string Standardized value, or a string containing an error
     */
    public static function execute($value, $mean, $stdDev)
    {
        $value = \ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Functions::flattenSingleValue($value);
        $mean = \ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Functions::flattenSingleValue($mean);
        $stdDev = \ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Functions::flattenSingleValue($stdDev);
        try {
            $value = self::validateFloat($value);
            $mean = self::validateFloat($mean);
            $stdDev = self::validateFloat($stdDev);
        } catch (\ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Exception $e) {
            return $e->getMessage();
        }
        if ($stdDev <= 0) {
            return \ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Functions::NAN();
        }
        return ($value - $mean) / $stdDev;
    }
}
