<?php

namespace ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\TextData;

use ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Calculation;
use ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Exception as CalcExp;
use ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Functions;
class Helpers
{
    public static function convertBooleanValue(bool $value) : string
    {
        if (\ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Functions::getCompatibilityMode() == \ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Functions::COMPATIBILITY_OPENOFFICE) {
            return $value ? '1' : '0';
        }
        return $value ? \ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Calculation::getTRUE() : \ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Calculation::getFALSE();
    }
    /**
     * @param mixed $value String value from which to extract characters
     */
    public static function extractString($value) : string
    {
        $value = \ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Functions::flattenSingleValue($value);
        if (\is_bool($value)) {
            return self::convertBooleanValue($value);
        }
        return (string) $value;
    }
    /**
     * @param mixed $value
     */
    public static function extractInt($value, int $minValue, int $gnumericNull = 0, bool $ooBoolOk = \false) : int
    {
        $value = \ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Functions::flattenSingleValue($value);
        if ($value === null) {
            // usually 0, but sometimes 1 for Gnumeric
            $value = \ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Functions::getCompatibilityMode() === \ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Functions::COMPATIBILITY_GNUMERIC ? $gnumericNull : 0;
        }
        if (\is_bool($value) && ($ooBoolOk || \ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Functions::getCompatibilityMode() !== \ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Functions::COMPATIBILITY_OPENOFFICE)) {
            $value = (int) $value;
        }
        if (!\is_numeric($value)) {
            throw new \ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Exception(\ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Functions::VALUE());
        }
        $value = (int) $value;
        if ($value < $minValue) {
            throw new \ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Exception(\ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Functions::VALUE());
        }
        return (int) $value;
    }
    /**
     * @param mixed $value
     */
    public static function extractFloat($value) : float
    {
        $value = \ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Functions::flattenSingleValue($value);
        if ($value === null) {
            $value = 0.0;
        }
        if (\is_bool($value)) {
            $value = (float) $value;
        }
        if (!\is_numeric($value)) {
            throw new \ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Exception(\ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Functions::VALUE());
        }
        return (float) $value;
    }
    /**
     * @param mixed $value
     */
    public static function validateInt($value) : int
    {
        $value = \ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Functions::flattenSingleValue($value);
        if ($value === null) {
            $value = 0;
        } elseif (\is_bool($value)) {
            $value = (int) $value;
        }
        return (int) $value;
    }
}
