<?php

namespace ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\TextData;

use ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Exception as CalcExp;
use ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Functions;
use ScopedGtbabel\PhpOffice\PhpSpreadsheet\Shared\StringHelper;
class Search
{
    /**
     * FIND (case sensitive search).
     *
     * @param mixed $needle The string to look for
     * @param mixed $haystack The string in which to look
     * @param mixed $offset Integer offset within $haystack to start searching from
     *
     * @return int|string
     */
    public static function sensitive($needle, $haystack, $offset = 1)
    {
        try {
            $needle = \ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\TextData\Helpers::extractString($needle);
            $haystack = \ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\TextData\Helpers::extractString($haystack);
            $offset = \ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\TextData\Helpers::extractInt($offset, 1, 0, \true);
        } catch (\ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Exception $e) {
            return $e->getMessage();
        }
        if (\ScopedGtbabel\PhpOffice\PhpSpreadsheet\Shared\StringHelper::countCharacters($haystack) >= $offset) {
            if (\ScopedGtbabel\PhpOffice\PhpSpreadsheet\Shared\StringHelper::countCharacters($needle) === 0) {
                return $offset;
            }
            $pos = \mb_strpos($haystack, $needle, --$offset, 'UTF-8');
            if ($pos !== \false) {
                return ++$pos;
            }
        }
        return \ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Functions::VALUE();
    }
    /**
     * SEARCH (case insensitive search).
     *
     * @param mixed $needle The string to look for
     * @param mixed $haystack The string in which to look
     * @param mixed $offset Integer offset within $haystack to start searching from
     *
     * @return int|string
     */
    public static function insensitive($needle, $haystack, $offset = 1)
    {
        try {
            $needle = \ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\TextData\Helpers::extractString($needle);
            $haystack = \ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\TextData\Helpers::extractString($haystack);
            $offset = \ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\TextData\Helpers::extractInt($offset, 1, 0, \true);
        } catch (\ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Exception $e) {
            return $e->getMessage();
        }
        if (\ScopedGtbabel\PhpOffice\PhpSpreadsheet\Shared\StringHelper::countCharacters($haystack) >= $offset) {
            if (\ScopedGtbabel\PhpOffice\PhpSpreadsheet\Shared\StringHelper::countCharacters($needle) === 0) {
                return $offset;
            }
            $pos = \mb_stripos($haystack, $needle, --$offset, 'UTF-8');
            if ($pos !== \false) {
                return ++$pos;
            }
        }
        return \ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Functions::VALUE();
    }
}
