<?php

namespace ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Web;

use ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Functions;
use ScopedGtbabel\PhpOffice\PhpSpreadsheet\Settings;
use ScopedGtbabel\Psr\Http\Client\ClientExceptionInterface;
class Service
{
    /**
     * WEBSERVICE.
     *
     * Returns data from a web service on the Internet or Intranet.
     *
     * Excel Function:
     *        Webservice(url)
     *
     * @return string the output resulting from a call to the webservice
     */
    public static function webService(string $url)
    {
        $url = \trim($url);
        if (\strlen($url) > 2048) {
            return \ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Functions::VALUE();
            // Invalid URL length
        }
        if (!\preg_match('/^http[s]?:\\/\\//', $url)) {
            return \ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Functions::VALUE();
            // Invalid protocol
        }
        // Get results from the the webservice
        $client = \ScopedGtbabel\PhpOffice\PhpSpreadsheet\Settings::getHttpClient();
        $requestFactory = \ScopedGtbabel\PhpOffice\PhpSpreadsheet\Settings::getRequestFactory();
        $request = $requestFactory->createRequest('GET', $url);
        try {
            $response = $client->sendRequest($request);
        } catch (\ScopedGtbabel\Psr\Http\Client\ClientExceptionInterface $e) {
            return \ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Functions::VALUE();
            // cURL error
        }
        if ($response->getStatusCode() != 200) {
            return \ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Functions::VALUE();
            // cURL error
        }
        $output = $response->getBody()->getContents();
        if (\strlen($output) > 32767) {
            return \ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Functions::VALUE();
            // Output not a string or too long
        }
        return $output;
    }
    /**
     * URLENCODE.
     *
     * Returns data from a web service on the Internet or Intranet.
     *
     * Excel Function:
     *        urlEncode(text)
     *
     * @param mixed $text
     *
     * @return string the url encoded output
     */
    public static function urlEncode($text)
    {
        if (!\is_string($text)) {
            return \ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Functions::VALUE();
        }
        return \str_replace('+', '%20', \urlencode($text));
    }
}
