<?php

namespace ScopedGtbabel\PhpOffice\PhpSpreadsheet\Reader\Xlsx;

use ScopedGtbabel\PhpOffice\PhpSpreadsheet\Cell\Coordinate;
use ScopedGtbabel\PhpOffice\PhpSpreadsheet\Reader\Xlsx;
use ScopedGtbabel\PhpOffice\PhpSpreadsheet\Worksheet\Worksheet;
use SimpleXMLElement;
class Hyperlinks
{
    private $worksheet;
    private $hyperlinks = [];
    public function __construct(\ScopedGtbabel\PhpOffice\PhpSpreadsheet\Worksheet\Worksheet $workSheet)
    {
        $this->worksheet = $workSheet;
    }
    public function readHyperlinks(\SimpleXMLElement $relsWorksheet) : void
    {
        foreach ($relsWorksheet->children(\ScopedGtbabel\PhpOffice\PhpSpreadsheet\Reader\Xlsx\Namespaces::RELATIONSHIPS)->Relationship as $elementx) {
            $element = \ScopedGtbabel\PhpOffice\PhpSpreadsheet\Reader\Xlsx::getAttributes($elementx);
            if ($element->Type == \ScopedGtbabel\PhpOffice\PhpSpreadsheet\Reader\Xlsx\Namespaces::HYPERLINK) {
                $this->hyperlinks[(string) $element->Id] = (string) $element->Target;
            }
        }
    }
    public function setHyperlinks(\SimpleXMLElement $worksheetXml) : void
    {
        foreach ($worksheetXml->children(\ScopedGtbabel\PhpOffice\PhpSpreadsheet\Reader\Xlsx\Namespaces::MAIN)->hyperlink as $hyperlink) {
            if ($hyperlink !== null) {
                $this->setHyperlink($hyperlink, $this->worksheet);
            }
        }
    }
    private function setHyperlink(\SimpleXMLElement $hyperlink, \ScopedGtbabel\PhpOffice\PhpSpreadsheet\Worksheet\Worksheet $worksheet) : void
    {
        // Link url
        $linkRel = \ScopedGtbabel\PhpOffice\PhpSpreadsheet\Reader\Xlsx::getAttributes($hyperlink, \ScopedGtbabel\PhpOffice\PhpSpreadsheet\Reader\Xlsx\Namespaces::SCHEMA_OFFICE_DOCUMENT);
        $attributes = \ScopedGtbabel\PhpOffice\PhpSpreadsheet\Reader\Xlsx::getAttributes($hyperlink);
        foreach (\ScopedGtbabel\PhpOffice\PhpSpreadsheet\Cell\Coordinate::extractAllCellReferencesInRange($attributes->ref) as $cellReference) {
            $cell = $worksheet->getCell($cellReference);
            if (isset($linkRel['id'])) {
                $hyperlinkUrl = $this->hyperlinks[(string) $linkRel['id']] ?? null;
                if (isset($hyperlink['location'])) {
                    $hyperlinkUrl .= '#' . (string) $hyperlink['location'];
                }
                $cell->getHyperlink()->setUrl($hyperlinkUrl);
            } elseif (isset($hyperlink['location'])) {
                $cell->getHyperlink()->setUrl('sheet://' . (string) $hyperlink['location']);
            }
            // Tooltip
            if (isset($hyperlink['tooltip'])) {
                $cell->getHyperlink()->setTooltip((string) $hyperlink['tooltip']);
            }
        }
    }
}
