<?php

namespace ScopedGtbabel\PhpOffice\PhpSpreadsheet\Reader\Xml\Style;

use ScopedGtbabel\PhpOffice\PhpSpreadsheet\Style\Fill as FillStyles;
use SimpleXMLElement;
class Fill extends \ScopedGtbabel\PhpOffice\PhpSpreadsheet\Reader\Xml\Style\StyleBase
{
    /**
     * @var array
     */
    public const FILL_MAPPINGS = ['fillType' => [
        'solid' => \ScopedGtbabel\PhpOffice\PhpSpreadsheet\Style\Fill::FILL_SOLID,
        'gray75' => \ScopedGtbabel\PhpOffice\PhpSpreadsheet\Style\Fill::FILL_PATTERN_DARKGRAY,
        'gray50' => \ScopedGtbabel\PhpOffice\PhpSpreadsheet\Style\Fill::FILL_PATTERN_MEDIUMGRAY,
        'gray25' => \ScopedGtbabel\PhpOffice\PhpSpreadsheet\Style\Fill::FILL_PATTERN_LIGHTGRAY,
        'gray125' => \ScopedGtbabel\PhpOffice\PhpSpreadsheet\Style\Fill::FILL_PATTERN_GRAY125,
        'gray0625' => \ScopedGtbabel\PhpOffice\PhpSpreadsheet\Style\Fill::FILL_PATTERN_GRAY0625,
        'horzstripe' => \ScopedGtbabel\PhpOffice\PhpSpreadsheet\Style\Fill::FILL_PATTERN_DARKHORIZONTAL,
        // horizontal stripe
        'vertstripe' => \ScopedGtbabel\PhpOffice\PhpSpreadsheet\Style\Fill::FILL_PATTERN_DARKVERTICAL,
        // vertical stripe
        'reversediagstripe' => \ScopedGtbabel\PhpOffice\PhpSpreadsheet\Style\Fill::FILL_PATTERN_DARKUP,
        // reverse diagonal stripe
        'diagstripe' => \ScopedGtbabel\PhpOffice\PhpSpreadsheet\Style\Fill::FILL_PATTERN_DARKDOWN,
        // diagonal stripe
        'diagcross' => \ScopedGtbabel\PhpOffice\PhpSpreadsheet\Style\Fill::FILL_PATTERN_DARKGRID,
        // diagoanl crosshatch
        'thickdiagcross' => \ScopedGtbabel\PhpOffice\PhpSpreadsheet\Style\Fill::FILL_PATTERN_DARKTRELLIS,
        // thick diagonal crosshatch
        'thinhorzstripe' => \ScopedGtbabel\PhpOffice\PhpSpreadsheet\Style\Fill::FILL_PATTERN_LIGHTHORIZONTAL,
        'thinvertstripe' => \ScopedGtbabel\PhpOffice\PhpSpreadsheet\Style\Fill::FILL_PATTERN_LIGHTVERTICAL,
        'thinreversediagstripe' => \ScopedGtbabel\PhpOffice\PhpSpreadsheet\Style\Fill::FILL_PATTERN_LIGHTUP,
        'thindiagstripe' => \ScopedGtbabel\PhpOffice\PhpSpreadsheet\Style\Fill::FILL_PATTERN_LIGHTDOWN,
        'thinhorzcross' => \ScopedGtbabel\PhpOffice\PhpSpreadsheet\Style\Fill::FILL_PATTERN_LIGHTGRID,
        // thin horizontal crosshatch
        'thindiagcross' => \ScopedGtbabel\PhpOffice\PhpSpreadsheet\Style\Fill::FILL_PATTERN_LIGHTTRELLIS,
    ]];
    public function parseStyle(\SimpleXMLElement $styleAttributes) : array
    {
        $style = [];
        foreach ($styleAttributes as $styleAttributeKey => $styleAttributeValuex) {
            $styleAttributeValue = (string) $styleAttributeValuex;
            switch ($styleAttributeKey) {
                case 'Color':
                    $style['fill']['endColor']['rgb'] = \substr($styleAttributeValue, 1);
                    $style['fill']['startColor']['rgb'] = \substr($styleAttributeValue, 1);
                    break;
                case 'PatternColor':
                    $style['fill']['startColor']['rgb'] = \substr($styleAttributeValue, 1);
                    break;
                case 'Pattern':
                    $lcStyleAttributeValue = \strtolower((string) $styleAttributeValue);
                    $style['fill']['fillType'] = self::FILL_MAPPINGS['fillType'][$lcStyleAttributeValue] ?? \ScopedGtbabel\PhpOffice\PhpSpreadsheet\Style\Fill::FILL_NONE;
                    break;
            }
        }
        return $style;
    }
}
