<?php

namespace ScopedGtbabel\PhpOffice\PhpSpreadsheet\Reader\Xml\Style;

use ScopedGtbabel\PhpOffice\PhpSpreadsheet\Style\Font as FontUnderline;
use SimpleXMLElement;
class Font extends \ScopedGtbabel\PhpOffice\PhpSpreadsheet\Reader\Xml\Style\StyleBase
{
    protected const UNDERLINE_STYLES = [\ScopedGtbabel\PhpOffice\PhpSpreadsheet\Style\Font::UNDERLINE_NONE, \ScopedGtbabel\PhpOffice\PhpSpreadsheet\Style\Font::UNDERLINE_DOUBLE, \ScopedGtbabel\PhpOffice\PhpSpreadsheet\Style\Font::UNDERLINE_DOUBLEACCOUNTING, \ScopedGtbabel\PhpOffice\PhpSpreadsheet\Style\Font::UNDERLINE_SINGLE, \ScopedGtbabel\PhpOffice\PhpSpreadsheet\Style\Font::UNDERLINE_SINGLEACCOUNTING];
    protected function parseUnderline(array $style, string $styleAttributeValue) : array
    {
        if (self::identifyFixedStyleValue(self::UNDERLINE_STYLES, $styleAttributeValue)) {
            $style['font']['underline'] = $styleAttributeValue;
        }
        return $style;
    }
    protected function parseVerticalAlign(array $style, string $styleAttributeValue) : array
    {
        if ($styleAttributeValue == 'Superscript') {
            $style['font']['superscript'] = \true;
        }
        if ($styleAttributeValue == 'Subscript') {
            $style['font']['subscript'] = \true;
        }
        return $style;
    }
    public function parseStyle(\SimpleXMLElement $styleAttributes) : array
    {
        $style = [];
        foreach ($styleAttributes as $styleAttributeKey => $styleAttributeValue) {
            $styleAttributeValue = (string) $styleAttributeValue;
            switch ($styleAttributeKey) {
                case 'FontName':
                    $style['font']['name'] = $styleAttributeValue;
                    break;
                case 'Size':
                    $style['font']['size'] = $styleAttributeValue;
                    break;
                case 'Color':
                    $style['font']['color']['rgb'] = \substr($styleAttributeValue, 1);
                    break;
                case 'Bold':
                    $style['font']['bold'] = \true;
                    break;
                case 'Italic':
                    $style['font']['italic'] = \true;
                    break;
                case 'Underline':
                    $style = $this->parseUnderline($style, $styleAttributeValue);
                    break;
                case 'VerticalAlign':
                    $style = $this->parseVerticalAlign($style, $styleAttributeValue);
                    break;
            }
        }
        return $style;
    }
}
