<?php

namespace ScopedGtbabel\PhpOffice\PhpSpreadsheet\Shared;

use ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Calculation;
class StringHelper
{
    /**    Constants                */
    /**    Regular Expressions        */
    //    Fraction
    const STRING_REGEXP_FRACTION = '(-?)(\\d+)\\s+(\\d+\\/\\d+)';
    /**
     * Control characters array.
     *
     * @var string[]
     */
    private static $controlCharacters = [];
    /**
     * SYLK Characters array.
     *
     * @var array
     */
    private static $SYLKCharacters = [];
    /**
     * Decimal separator.
     *
     * @var string
     */
    private static $decimalSeparator;
    /**
     * Thousands separator.
     *
     * @var string
     */
    private static $thousandsSeparator;
    /**
     * Currency code.
     *
     * @var string
     */
    private static $currencyCode;
    /**
     * Is iconv extension avalable?
     *
     * @var bool
     */
    private static $isIconvEnabled;
    /**
     * iconv options.
     *
     * @var string
     */
    private static $iconvOptions = '//IGNORE//TRANSLIT';
    /**
     * Build control characters array.
     */
    private static function buildControlCharacters() : void
    {
        for ($i = 0; $i <= 31; ++$i) {
            if ($i != 9 && $i != 10 && $i != 13) {
                $find = '_x' . \sprintf('%04s', \strtoupper(\dechex($i))) . '_';
                $replace = \chr($i);
                self::$controlCharacters[$find] = $replace;
            }
        }
    }
    /**
     * Build SYLK characters array.
     */
    private static function buildSYLKCharacters() : void
    {
        self::$SYLKCharacters = [
            "\33 0" => \chr(0),
            "\33 1" => \chr(1),
            "\33 2" => \chr(2),
            "\33 3" => \chr(3),
            "\33 4" => \chr(4),
            "\33 5" => \chr(5),
            "\33 6" => \chr(6),
            "\33 7" => \chr(7),
            "\33 8" => \chr(8),
            "\33 9" => \chr(9),
            "\33 :" => \chr(10),
            "\33 ;" => \chr(11),
            "\33 <" => \chr(12),
            "\33 =" => \chr(13),
            "\33 >" => \chr(14),
            "\33 ?" => \chr(15),
            "\33!0" => \chr(16),
            "\33!1" => \chr(17),
            "\33!2" => \chr(18),
            "\33!3" => \chr(19),
            "\33!4" => \chr(20),
            "\33!5" => \chr(21),
            "\33!6" => \chr(22),
            "\33!7" => \chr(23),
            "\33!8" => \chr(24),
            "\33!9" => \chr(25),
            "\33!:" => \chr(26),
            "\33!;" => \chr(27),
            "\33!<" => \chr(28),
            "\33!=" => \chr(29),
            "\33!>" => \chr(30),
            "\33!?" => \chr(31),
            "\33'?" => \chr(127),
            "\33(0" => '€',
            // 128 in CP1252
            "\33(2" => '‚',
            // 130 in CP1252
            "\33(3" => 'ƒ',
            // 131 in CP1252
            "\33(4" => '„',
            // 132 in CP1252
            "\33(5" => '…',
            // 133 in CP1252
            "\33(6" => '†',
            // 134 in CP1252
            "\33(7" => '‡',
            // 135 in CP1252
            "\33(8" => 'ˆ',
            // 136 in CP1252
            "\33(9" => '‰',
            // 137 in CP1252
            "\33(:" => 'Š',
            // 138 in CP1252
            "\33(;" => '‹',
            // 139 in CP1252
            "\33Nj" => 'Œ',
            // 140 in CP1252
            "\33(>" => 'Ž',
            // 142 in CP1252
            "\33)1" => '‘',
            // 145 in CP1252
            "\33)2" => '’',
            // 146 in CP1252
            "\33)3" => '“',
            // 147 in CP1252
            "\33)4" => '”',
            // 148 in CP1252
            "\33)5" => '•',
            // 149 in CP1252
            "\33)6" => '–',
            // 150 in CP1252
            "\33)7" => '—',
            // 151 in CP1252
            "\33)8" => '˜',
            // 152 in CP1252
            "\33)9" => '™',
            // 153 in CP1252
            "\33):" => 'š',
            // 154 in CP1252
            "\33);" => '›',
            // 155 in CP1252
            "\33Nz" => 'œ',
            // 156 in CP1252
            "\33)>" => 'ž',
            // 158 in CP1252
            "\33)?" => 'Ÿ',
            // 159 in CP1252
            "\33*0" => ' ',
            // 160 in CP1252
            "\33N!" => '¡',
            // 161 in CP1252
            "\33N\"" => '¢',
            // 162 in CP1252
            "\33N#" => '£',
            // 163 in CP1252
            "\33N(" => '¤',
            // 164 in CP1252
            "\33N%" => '¥',
            // 165 in CP1252
            "\33*6" => '¦',
            // 166 in CP1252
            "\33N'" => '§',
            // 167 in CP1252
            "\33NH " => '¨',
            // 168 in CP1252
            "\33NS" => '©',
            // 169 in CP1252
            "\33Nc" => 'ª',
            // 170 in CP1252
            "\33N+" => '«',
            // 171 in CP1252
            "\33*<" => '¬',
            // 172 in CP1252
            "\33*=" => '­',
            // 173 in CP1252
            "\33NR" => '®',
            // 174 in CP1252
            "\33*?" => '¯',
            // 175 in CP1252
            "\33N0" => '°',
            // 176 in CP1252
            "\33N1" => '±',
            // 177 in CP1252
            "\33N2" => '²',
            // 178 in CP1252
            "\33N3" => '³',
            // 179 in CP1252
            "\33NB " => '´',
            // 180 in CP1252
            "\33N5" => 'µ',
            // 181 in CP1252
            "\33N6" => '¶',
            // 182 in CP1252
            "\33N7" => '·',
            // 183 in CP1252
            "\33+8" => '¸',
            // 184 in CP1252
            "\33NQ" => '¹',
            // 185 in CP1252
            "\33Nk" => 'º',
            // 186 in CP1252
            "\33N;" => '»',
            // 187 in CP1252
            "\33N<" => '¼',
            // 188 in CP1252
            "\33N=" => '½',
            // 189 in CP1252
            "\33N>" => '¾',
            // 190 in CP1252
            "\33N?" => '¿',
            // 191 in CP1252
            "\33NAA" => 'À',
            // 192 in CP1252
            "\33NBA" => 'Á',
            // 193 in CP1252
            "\33NCA" => 'Â',
            // 194 in CP1252
            "\33NDA" => 'Ã',
            // 195 in CP1252
            "\33NHA" => 'Ä',
            // 196 in CP1252
            "\33NJA" => 'Å',
            // 197 in CP1252
            "\33Na" => 'Æ',
            // 198 in CP1252
            "\33NKC" => 'Ç',
            // 199 in CP1252
            "\33NAE" => 'È',
            // 200 in CP1252
            "\33NBE" => 'É',
            // 201 in CP1252
            "\33NCE" => 'Ê',
            // 202 in CP1252
            "\33NHE" => 'Ë',
            // 203 in CP1252
            "\33NAI" => 'Ì',
            // 204 in CP1252
            "\33NBI" => 'Í',
            // 205 in CP1252
            "\33NCI" => 'Î',
            // 206 in CP1252
            "\33NHI" => 'Ï',
            // 207 in CP1252
            "\33Nb" => 'Ð',
            // 208 in CP1252
            "\33NDN" => 'Ñ',
            // 209 in CP1252
            "\33NAO" => 'Ò',
            // 210 in CP1252
            "\33NBO" => 'Ó',
            // 211 in CP1252
            "\33NCO" => 'Ô',
            // 212 in CP1252
            "\33NDO" => 'Õ',
            // 213 in CP1252
            "\33NHO" => 'Ö',
            // 214 in CP1252
            "\33-7" => '×',
            // 215 in CP1252
            "\33Ni" => 'Ø',
            // 216 in CP1252
            "\33NAU" => 'Ù',
            // 217 in CP1252
            "\33NBU" => 'Ú',
            // 218 in CP1252
            "\33NCU" => 'Û',
            // 219 in CP1252
            "\33NHU" => 'Ü',
            // 220 in CP1252
            "\33-=" => 'Ý',
            // 221 in CP1252
            "\33Nl" => 'Þ',
            // 222 in CP1252
            "\33N{" => 'ß',
            // 223 in CP1252
            "\33NAa" => 'à',
            // 224 in CP1252
            "\33NBa" => 'á',
            // 225 in CP1252
            "\33NCa" => 'â',
            // 226 in CP1252
            "\33NDa" => 'ã',
            // 227 in CP1252
            "\33NHa" => 'ä',
            // 228 in CP1252
            "\33NJa" => 'å',
            // 229 in CP1252
            "\33Nq" => 'æ',
            // 230 in CP1252
            "\33NKc" => 'ç',
            // 231 in CP1252
            "\33NAe" => 'è',
            // 232 in CP1252
            "\33NBe" => 'é',
            // 233 in CP1252
            "\33NCe" => 'ê',
            // 234 in CP1252
            "\33NHe" => 'ë',
            // 235 in CP1252
            "\33NAi" => 'ì',
            // 236 in CP1252
            "\33NBi" => 'í',
            // 237 in CP1252
            "\33NCi" => 'î',
            // 238 in CP1252
            "\33NHi" => 'ï',
            // 239 in CP1252
            "\33Ns" => 'ð',
            // 240 in CP1252
            "\33NDn" => 'ñ',
            // 241 in CP1252
            "\33NAo" => 'ò',
            // 242 in CP1252
            "\33NBo" => 'ó',
            // 243 in CP1252
            "\33NCo" => 'ô',
            // 244 in CP1252
            "\33NDo" => 'õ',
            // 245 in CP1252
            "\33NHo" => 'ö',
            // 246 in CP1252
            "\33/7" => '÷',
            // 247 in CP1252
            "\33Ny" => 'ø',
            // 248 in CP1252
            "\33NAu" => 'ù',
            // 249 in CP1252
            "\33NBu" => 'ú',
            // 250 in CP1252
            "\33NCu" => 'û',
            // 251 in CP1252
            "\33NHu" => 'ü',
            // 252 in CP1252
            "\33/=" => 'ý',
            // 253 in CP1252
            "\33N|" => 'þ',
            // 254 in CP1252
            "\33NHy" => 'ÿ',
        ];
    }
    /**
     * Get whether iconv extension is available.
     *
     * @return bool
     */
    public static function getIsIconvEnabled()
    {
        if (isset(self::$isIconvEnabled)) {
            return self::$isIconvEnabled;
        }
        // Assume no problems with iconv
        self::$isIconvEnabled = \true;
        // Fail if iconv doesn't exist
        if (!\function_exists('iconv')) {
            self::$isIconvEnabled = \false;
        } elseif (!@\iconv('UTF-8', 'UTF-16LE', 'x')) {
            // Sometimes iconv is not working, and e.g. iconv('UTF-8', 'UTF-16LE', 'x') just returns false,
            self::$isIconvEnabled = \false;
        } elseif (\defined('PHP_OS') && @\stristr(\PHP_OS, 'AIX') && \defined('ICONV_IMPL') && @\strcasecmp(\ICONV_IMPL, 'unknown') == 0 && \defined('ICONV_VERSION') && @\strcasecmp(\ICONV_VERSION, 'unknown') == 0) {
            // CUSTOM: IBM AIX iconv() does not work
            self::$isIconvEnabled = \false;
        }
        // Deactivate iconv default options if they fail (as seen on IMB i)
        if (self::$isIconvEnabled && !@\iconv('UTF-8', 'UTF-16LE' . self::$iconvOptions, 'x')) {
            self::$iconvOptions = '';
        }
        return self::$isIconvEnabled;
    }
    private static function buildCharacterSets() : void
    {
        if (empty(self::$controlCharacters)) {
            self::buildControlCharacters();
        }
        if (empty(self::$SYLKCharacters)) {
            self::buildSYLKCharacters();
        }
    }
    /**
     * Convert from OpenXML escaped control character to PHP control character.
     *
     * Excel 2007 team:
     * ----------------
     * That's correct, control characters are stored directly in the shared-strings table.
     * We do encode characters that cannot be represented in XML using the following escape sequence:
     * _xHHHH_ where H represents a hexadecimal character in the character's value...
     * So you could end up with something like _x0008_ in a string (either in a cell value (<v>)
     * element or in the shared string <t> element.
     *
     * @param string $textValue Value to unescape
     *
     * @return string
     */
    public static function controlCharacterOOXML2PHP($textValue)
    {
        self::buildCharacterSets();
        return \str_replace(\array_keys(self::$controlCharacters), \array_values(self::$controlCharacters), $textValue);
    }
    /**
     * Convert from PHP control character to OpenXML escaped control character.
     *
     * Excel 2007 team:
     * ----------------
     * That's correct, control characters are stored directly in the shared-strings table.
     * We do encode characters that cannot be represented in XML using the following escape sequence:
     * _xHHHH_ where H represents a hexadecimal character in the character's value...
     * So you could end up with something like _x0008_ in a string (either in a cell value (<v>)
     * element or in the shared string <t> element.
     *
     * @param string $textValue Value to escape
     *
     * @return string
     */
    public static function controlCharacterPHP2OOXML($textValue)
    {
        self::buildCharacterSets();
        return \str_replace(\array_values(self::$controlCharacters), \array_keys(self::$controlCharacters), $textValue);
    }
    /**
     * Try to sanitize UTF8, stripping invalid byte sequences. Not perfect. Does not surrogate characters.
     *
     * @param string $textValue
     *
     * @return string
     */
    public static function sanitizeUTF8($textValue)
    {
        if (self::getIsIconvEnabled()) {
            $textValue = @\iconv('UTF-8', 'UTF-8', $textValue);
            return $textValue;
        }
        $textValue = \mb_convert_encoding($textValue, 'UTF-8', 'UTF-8');
        return $textValue;
    }
    /**
     * Check if a string contains UTF8 data.
     *
     * @param string $textValue
     *
     * @return bool
     */
    public static function isUTF8($textValue)
    {
        return $textValue === '' || \preg_match('/^./su', $textValue) === 1;
    }
    /**
     * Formats a numeric value as a string for output in various output writers forcing
     * point as decimal separator in case locale is other than English.
     *
     * @param mixed $numericValue
     *
     * @return string
     */
    public static function formatNumber($numericValue)
    {
        if (\is_float($numericValue)) {
            return \str_replace(',', '.', $numericValue);
        }
        return (string) $numericValue;
    }
    /**
     * Converts a UTF-8 string into BIFF8 Unicode string data (8-bit string length)
     * Writes the string using uncompressed notation, no rich text, no Asian phonetics
     * If mbstring extension is not available, ASCII is assumed, and compressed notation is used
     * although this will give wrong results for non-ASCII strings
     * see OpenOffice.org's Documentation of the Microsoft Excel File Format, sect. 2.5.3.
     *
     * @param string $textValue UTF-8 encoded string
     * @param mixed[] $arrcRuns Details of rich text runs in $value
     *
     * @return string
     */
    public static function UTF8toBIFF8UnicodeShort($textValue, $arrcRuns = [])
    {
        // character count
        $ln = self::countCharacters($textValue, 'UTF-8');
        // option flags
        if (empty($arrcRuns)) {
            $data = \pack('CC', $ln, 0x1);
            // characters
            $data .= self::convertEncoding($textValue, 'UTF-16LE', 'UTF-8');
        } else {
            $data = \pack('vC', $ln, 0x9);
            $data .= \pack('v', \count($arrcRuns));
            // characters
            $data .= self::convertEncoding($textValue, 'UTF-16LE', 'UTF-8');
            foreach ($arrcRuns as $cRun) {
                $data .= \pack('v', $cRun['strlen']);
                $data .= \pack('v', $cRun['fontidx']);
            }
        }
        return $data;
    }
    /**
     * Converts a UTF-8 string into BIFF8 Unicode string data (16-bit string length)
     * Writes the string using uncompressed notation, no rich text, no Asian phonetics
     * If mbstring extension is not available, ASCII is assumed, and compressed notation is used
     * although this will give wrong results for non-ASCII strings
     * see OpenOffice.org's Documentation of the Microsoft Excel File Format, sect. 2.5.3.
     *
     * @param string $textValue UTF-8 encoded string
     *
     * @return string
     */
    public static function UTF8toBIFF8UnicodeLong($textValue)
    {
        // character count
        $ln = self::countCharacters($textValue, 'UTF-8');
        // characters
        $chars = self::convertEncoding($textValue, 'UTF-16LE', 'UTF-8');
        return \pack('vC', $ln, 0x1) . $chars;
    }
    /**
     * Convert string from one encoding to another.
     *
     * @param string $textValue
     * @param string $to Encoding to convert to, e.g. 'UTF-8'
     * @param string $from Encoding to convert from, e.g. 'UTF-16LE'
     *
     * @return string
     */
    public static function convertEncoding($textValue, $to, $from)
    {
        if (self::getIsIconvEnabled()) {
            $result = \iconv($from, $to . self::$iconvOptions, $textValue);
            if (\false !== $result) {
                return $result;
            }
        }
        return \mb_convert_encoding($textValue, $to, $from);
    }
    /**
     * Get character count.
     *
     * @param string $textValue
     * @param string $encoding Encoding
     *
     * @return int Character count
     */
    public static function countCharacters($textValue, $encoding = 'UTF-8')
    {
        return \mb_strlen($textValue ?? '', $encoding);
    }
    /**
     * Get a substring of a UTF-8 encoded string.
     *
     * @param string $textValue UTF-8 encoded string
     * @param int $offset Start offset
     * @param int $length Maximum number of characters in substring
     *
     * @return string
     */
    public static function substring($textValue, $offset, $length = 0)
    {
        return \mb_substr($textValue, $offset, $length, 'UTF-8');
    }
    /**
     * Convert a UTF-8 encoded string to upper case.
     *
     * @param string $textValue UTF-8 encoded string
     *
     * @return string
     */
    public static function strToUpper($textValue)
    {
        return \mb_convert_case($textValue, \MB_CASE_UPPER, 'UTF-8');
    }
    /**
     * Convert a UTF-8 encoded string to lower case.
     *
     * @param string $textValue UTF-8 encoded string
     *
     * @return string
     */
    public static function strToLower($textValue)
    {
        return \mb_convert_case($textValue ?? '', \MB_CASE_LOWER, 'UTF-8');
    }
    /**
     * Convert a UTF-8 encoded string to title/proper case
     * (uppercase every first character in each word, lower case all other characters).
     *
     * @param string $textValue UTF-8 encoded string
     *
     * @return string
     */
    public static function strToTitle($textValue)
    {
        return \mb_convert_case($textValue, \MB_CASE_TITLE, 'UTF-8');
    }
    public static function mbIsUpper($character)
    {
        return \mb_strtolower($character, 'UTF-8') != $character;
    }
    public static function mbStrSplit($string)
    {
        // Split at all position not after the start: ^
        // and not before the end: $
        return \preg_split('/(?<!^)(?!$)/u', $string);
    }
    /**
     * Reverse the case of a string, so that all uppercase characters become lowercase
     * and all lowercase characters become uppercase.
     *
     * @param string $textValue UTF-8 encoded string
     *
     * @return string
     */
    public static function strCaseReverse($textValue)
    {
        $characters = self::mbStrSplit($textValue);
        foreach ($characters as &$character) {
            if (self::mbIsUpper($character)) {
                $character = \mb_strtolower($character, 'UTF-8');
            } else {
                $character = \mb_strtoupper($character, 'UTF-8');
            }
        }
        return \implode('', $characters);
    }
    /**
     * Identify whether a string contains a fractional numeric value,
     * and convert it to a numeric if it is.
     *
     * @param string $operand string value to test
     *
     * @return bool
     */
    public static function convertToNumberIfFraction(&$operand)
    {
        if (\preg_match('/^' . self::STRING_REGEXP_FRACTION . '$/i', $operand, $match)) {
            $sign = $match[1] == '-' ? '-' : '+';
            $fractionFormula = '=' . $sign . $match[2] . $sign . $match[3];
            $operand = \ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Calculation::getInstance()->_calculateFormulaValue($fractionFormula);
            return \true;
        }
        return \false;
    }
    //    function convertToNumberIfFraction()
    /**
     * Get the decimal separator. If it has not yet been set explicitly, try to obtain number
     * formatting information from locale.
     *
     * @return string
     */
    public static function getDecimalSeparator()
    {
        if (!isset(self::$decimalSeparator)) {
            $localeconv = \localeconv();
            self::$decimalSeparator = $localeconv['decimal_point'] != '' ? $localeconv['decimal_point'] : $localeconv['mon_decimal_point'];
            if (self::$decimalSeparator == '') {
                // Default to .
                self::$decimalSeparator = '.';
            }
        }
        return self::$decimalSeparator;
    }
    /**
     * Set the decimal separator. Only used by NumberFormat::toFormattedString()
     * to format output by \PhpOffice\PhpSpreadsheet\Writer\Html and \PhpOffice\PhpSpreadsheet\Writer\Pdf.
     *
     * @param string $separator Character for decimal separator
     */
    public static function setDecimalSeparator($separator) : void
    {
        self::$decimalSeparator = $separator;
    }
    /**
     * Get the thousands separator. If it has not yet been set explicitly, try to obtain number
     * formatting information from locale.
     *
     * @return string
     */
    public static function getThousandsSeparator()
    {
        if (!isset(self::$thousandsSeparator)) {
            $localeconv = \localeconv();
            self::$thousandsSeparator = $localeconv['thousands_sep'] != '' ? $localeconv['thousands_sep'] : $localeconv['mon_thousands_sep'];
            if (self::$thousandsSeparator == '') {
                // Default to .
                self::$thousandsSeparator = ',';
            }
        }
        return self::$thousandsSeparator;
    }
    /**
     * Set the thousands separator. Only used by NumberFormat::toFormattedString()
     * to format output by \PhpOffice\PhpSpreadsheet\Writer\Html and \PhpOffice\PhpSpreadsheet\Writer\Pdf.
     *
     * @param string $separator Character for thousands separator
     */
    public static function setThousandsSeparator($separator) : void
    {
        self::$thousandsSeparator = $separator;
    }
    /**
     *    Get the currency code. If it has not yet been set explicitly, try to obtain the
     *        symbol information from locale.
     *
     * @return string
     */
    public static function getCurrencyCode()
    {
        if (!empty(self::$currencyCode)) {
            return self::$currencyCode;
        }
        self::$currencyCode = '$';
        $localeconv = \localeconv();
        if (!empty($localeconv['currency_symbol'])) {
            self::$currencyCode = $localeconv['currency_symbol'];
            return self::$currencyCode;
        }
        if (!empty($localeconv['int_curr_symbol'])) {
            self::$currencyCode = $localeconv['int_curr_symbol'];
            return self::$currencyCode;
        }
        return self::$currencyCode;
    }
    /**
     * Set the currency code. Only used by NumberFormat::toFormattedString()
     *        to format output by \PhpOffice\PhpSpreadsheet\Writer\Html and \PhpOffice\PhpSpreadsheet\Writer\Pdf.
     *
     * @param string $currencyCode Character for currency code
     */
    public static function setCurrencyCode($currencyCode) : void
    {
        self::$currencyCode = $currencyCode;
    }
    /**
     * Convert SYLK encoded string to UTF-8.
     *
     * @param string $textValue
     *
     * @return string UTF-8 encoded string
     */
    public static function SYLKtoUTF8($textValue)
    {
        self::buildCharacterSets();
        // If there is no escape character in the string there is nothing to do
        if (\strpos($textValue, '') === \false) {
            return $textValue;
        }
        foreach (self::$SYLKCharacters as $k => $v) {
            $textValue = \str_replace($k, $v, $textValue);
        }
        return $textValue;
    }
    /**
     * Retrieve any leading numeric part of a string, or return the full string if no leading numeric
     * (handles basic integer or float, but not exponent or non decimal).
     *
     * @param string $textValue
     *
     * @return mixed string or only the leading numeric part of the string
     */
    public static function testStringAsNumeric($textValue)
    {
        if (\is_numeric($textValue)) {
            return $textValue;
        }
        $v = (float) $textValue;
        return \is_numeric(\substr($textValue, 0, \strlen($v))) ? $v : $textValue;
    }
}
