<?php

namespace ScopedGtbabel\PhpOffice\PhpSpreadsheet\Writer;

use ScopedGtbabel\PhpOffice\PhpSpreadsheet\Spreadsheet;
use ScopedGtbabel\PhpOffice\PhpSpreadsheet\Writer\Exception as WriterException;
use ScopedGtbabel\PhpOffice\PhpSpreadsheet\Writer\Ods\Content;
use ScopedGtbabel\PhpOffice\PhpSpreadsheet\Writer\Ods\Meta;
use ScopedGtbabel\PhpOffice\PhpSpreadsheet\Writer\Ods\MetaInf;
use ScopedGtbabel\PhpOffice\PhpSpreadsheet\Writer\Ods\Mimetype;
use ScopedGtbabel\PhpOffice\PhpSpreadsheet\Writer\Ods\Settings;
use ScopedGtbabel\PhpOffice\PhpSpreadsheet\Writer\Ods\Styles;
use ScopedGtbabel\PhpOffice\PhpSpreadsheet\Writer\Ods\Thumbnails;
use ScopedGtbabel\ZipStream\Exception\OverflowException;
use ScopedGtbabel\ZipStream\Option\Archive;
use ScopedGtbabel\ZipStream\ZipStream;
class Ods extends \ScopedGtbabel\PhpOffice\PhpSpreadsheet\Writer\BaseWriter
{
    /**
     * Private PhpSpreadsheet.
     *
     * @var Spreadsheet
     */
    private $spreadSheet;
    /**
     * @var Content
     */
    private $writerPartContent;
    /**
     * @var Meta
     */
    private $writerPartMeta;
    /**
     * @var MetaInf
     */
    private $writerPartMetaInf;
    /**
     * @var Mimetype
     */
    private $writerPartMimetype;
    /**
     * @var Settings
     */
    private $writerPartSettings;
    /**
     * @var Styles
     */
    private $writerPartStyles;
    /**
     * @var Thumbnails
     */
    private $writerPartThumbnails;
    /**
     * Create a new Ods.
     */
    public function __construct(\ScopedGtbabel\PhpOffice\PhpSpreadsheet\Spreadsheet $spreadsheet)
    {
        $this->setSpreadsheet($spreadsheet);
        $this->writerPartContent = new \ScopedGtbabel\PhpOffice\PhpSpreadsheet\Writer\Ods\Content($this);
        $this->writerPartMeta = new \ScopedGtbabel\PhpOffice\PhpSpreadsheet\Writer\Ods\Meta($this);
        $this->writerPartMetaInf = new \ScopedGtbabel\PhpOffice\PhpSpreadsheet\Writer\Ods\MetaInf($this);
        $this->writerPartMimetype = new \ScopedGtbabel\PhpOffice\PhpSpreadsheet\Writer\Ods\Mimetype($this);
        $this->writerPartSettings = new \ScopedGtbabel\PhpOffice\PhpSpreadsheet\Writer\Ods\Settings($this);
        $this->writerPartStyles = new \ScopedGtbabel\PhpOffice\PhpSpreadsheet\Writer\Ods\Styles($this);
        $this->writerPartThumbnails = new \ScopedGtbabel\PhpOffice\PhpSpreadsheet\Writer\Ods\Thumbnails($this);
    }
    public function getWriterPartContent() : \ScopedGtbabel\PhpOffice\PhpSpreadsheet\Writer\Ods\Content
    {
        return $this->writerPartContent;
    }
    public function getWriterPartMeta() : \ScopedGtbabel\PhpOffice\PhpSpreadsheet\Writer\Ods\Meta
    {
        return $this->writerPartMeta;
    }
    public function getWriterPartMetaInf() : \ScopedGtbabel\PhpOffice\PhpSpreadsheet\Writer\Ods\MetaInf
    {
        return $this->writerPartMetaInf;
    }
    public function getWriterPartMimetype() : \ScopedGtbabel\PhpOffice\PhpSpreadsheet\Writer\Ods\Mimetype
    {
        return $this->writerPartMimetype;
    }
    public function getWriterPartSettings() : \ScopedGtbabel\PhpOffice\PhpSpreadsheet\Writer\Ods\Settings
    {
        return $this->writerPartSettings;
    }
    public function getWriterPartStyles() : \ScopedGtbabel\PhpOffice\PhpSpreadsheet\Writer\Ods\Styles
    {
        return $this->writerPartStyles;
    }
    public function getWriterPartThumbnails() : \ScopedGtbabel\PhpOffice\PhpSpreadsheet\Writer\Ods\Thumbnails
    {
        return $this->writerPartThumbnails;
    }
    /**
     * Save PhpSpreadsheet to file.
     *
     * @param resource|string $filename
     */
    public function save($filename, int $flags = 0) : void
    {
        if (!$this->spreadSheet) {
            throw new \ScopedGtbabel\PhpOffice\PhpSpreadsheet\Writer\Exception('PhpSpreadsheet object unassigned.');
        }
        $this->processFlags($flags);
        // garbage collect
        $this->spreadSheet->garbageCollect();
        $this->openFileHandle($filename);
        $zip = $this->createZip();
        $zip->addFile('META-INF/manifest.xml', $this->getWriterPartMetaInf()->write());
        $zip->addFile('Thumbnails/thumbnail.png', $this->getWriterPartthumbnails()->write());
        $zip->addFile('content.xml', $this->getWriterPartcontent()->write());
        $zip->addFile('meta.xml', $this->getWriterPartmeta()->write());
        $zip->addFile('mimetype', $this->getWriterPartmimetype()->write());
        $zip->addFile('settings.xml', $this->getWriterPartsettings()->write());
        $zip->addFile('styles.xml', $this->getWriterPartstyles()->write());
        // Close file
        try {
            $zip->finish();
        } catch (\ScopedGtbabel\ZipStream\Exception\OverflowException $e) {
            throw new \ScopedGtbabel\PhpOffice\PhpSpreadsheet\Writer\Exception('Could not close resource.');
        }
        $this->maybeCloseFileHandle();
    }
    /**
     * Create zip object.
     *
     * @return ZipStream
     */
    private function createZip()
    {
        // Try opening the ZIP file
        if (!\is_resource($this->fileHandle)) {
            throw new \ScopedGtbabel\PhpOffice\PhpSpreadsheet\Writer\Exception('Could not open resource for writing.');
        }
        // Create new ZIP stream
        $options = new \ScopedGtbabel\ZipStream\Option\Archive();
        $options->setEnableZip64(\false);
        $options->setOutputStream($this->fileHandle);
        return new \ScopedGtbabel\ZipStream\ZipStream(null, $options);
    }
    /**
     * Get Spreadsheet object.
     *
     * @return Spreadsheet
     */
    public function getSpreadsheet()
    {
        if ($this->spreadSheet !== null) {
            return $this->spreadSheet;
        }
        throw new \ScopedGtbabel\PhpOffice\PhpSpreadsheet\Writer\Exception('No PhpSpreadsheet assigned.');
    }
    /**
     * Set Spreadsheet object.
     *
     * @param Spreadsheet $spreadsheet PhpSpreadsheet object
     *
     * @return $this
     */
    public function setSpreadsheet(\ScopedGtbabel\PhpOffice\PhpSpreadsheet\Spreadsheet $spreadsheet)
    {
        $this->spreadSheet = $spreadsheet;
        return $this;
    }
}
