<?php

namespace ScopedGtbabel\PhpOffice\PhpSpreadsheet\Writer\Ods\Cell;

use ScopedGtbabel\PhpOffice\PhpSpreadsheet\Shared\XMLWriter;
use ScopedGtbabel\PhpOffice\PhpSpreadsheet\Style\Alignment;
use ScopedGtbabel\PhpOffice\PhpSpreadsheet\Style\Fill;
use ScopedGtbabel\PhpOffice\PhpSpreadsheet\Style\Font;
use ScopedGtbabel\PhpOffice\PhpSpreadsheet\Style\Style as CellStyle;
class Style
{
    public const CELL_STYLE_PREFIX = 'ce';
    private $writer;
    public function __construct(\ScopedGtbabel\PhpOffice\PhpSpreadsheet\Shared\XMLWriter $writer)
    {
        $this->writer = $writer;
    }
    private function mapHorizontalAlignment(string $horizontalAlignment) : string
    {
        switch ($horizontalAlignment) {
            case \ScopedGtbabel\PhpOffice\PhpSpreadsheet\Style\Alignment::HORIZONTAL_CENTER:
            case \ScopedGtbabel\PhpOffice\PhpSpreadsheet\Style\Alignment::HORIZONTAL_CENTER_CONTINUOUS:
            case \ScopedGtbabel\PhpOffice\PhpSpreadsheet\Style\Alignment::HORIZONTAL_DISTRIBUTED:
                return 'center';
            case \ScopedGtbabel\PhpOffice\PhpSpreadsheet\Style\Alignment::HORIZONTAL_RIGHT:
                return 'end';
            case \ScopedGtbabel\PhpOffice\PhpSpreadsheet\Style\Alignment::HORIZONTAL_FILL:
            case \ScopedGtbabel\PhpOffice\PhpSpreadsheet\Style\Alignment::HORIZONTAL_JUSTIFY:
                return 'justify';
        }
        return 'start';
    }
    private function mapVerticalAlignment(string $verticalAlignment) : string
    {
        switch ($verticalAlignment) {
            case \ScopedGtbabel\PhpOffice\PhpSpreadsheet\Style\Alignment::VERTICAL_TOP:
                return 'top';
            case \ScopedGtbabel\PhpOffice\PhpSpreadsheet\Style\Alignment::VERTICAL_CENTER:
                return 'middle';
            case \ScopedGtbabel\PhpOffice\PhpSpreadsheet\Style\Alignment::VERTICAL_DISTRIBUTED:
            case \ScopedGtbabel\PhpOffice\PhpSpreadsheet\Style\Alignment::VERTICAL_JUSTIFY:
                return 'automatic';
        }
        return 'bottom';
    }
    private function writeFillStyle(\ScopedGtbabel\PhpOffice\PhpSpreadsheet\Style\Fill $fill) : void
    {
        switch ($fill->getFillType()) {
            case \ScopedGtbabel\PhpOffice\PhpSpreadsheet\Style\Fill::FILL_SOLID:
                $this->writer->writeAttribute('fo:background-color', \sprintf('#%s', \strtolower($fill->getStartColor()->getRGB())));
                break;
            case \ScopedGtbabel\PhpOffice\PhpSpreadsheet\Style\Fill::FILL_GRADIENT_LINEAR:
            case \ScopedGtbabel\PhpOffice\PhpSpreadsheet\Style\Fill::FILL_GRADIENT_PATH:
                /// TODO :: To be implemented
                break;
            case \ScopedGtbabel\PhpOffice\PhpSpreadsheet\Style\Fill::FILL_NONE:
            default:
        }
    }
    private function writeCellProperties(\ScopedGtbabel\PhpOffice\PhpSpreadsheet\Style\Style $style) : void
    {
        // Align
        $hAlign = $style->getAlignment()->getHorizontal();
        $vAlign = $style->getAlignment()->getVertical();
        $wrap = $style->getAlignment()->getWrapText();
        $this->writer->startElement('style:table-cell-properties');
        if (!empty($vAlign) || $wrap) {
            if (!empty($vAlign)) {
                $vAlign = $this->mapVerticalAlignment($vAlign);
                $this->writer->writeAttribute('style:vertical-align', $vAlign);
            }
            if ($wrap) {
                $this->writer->writeAttribute('fo:wrap-option', 'wrap');
            }
        }
        $this->writer->writeAttribute('style:rotation-align', 'none');
        // Fill
        if ($fill = $style->getFill()) {
            $this->writeFillStyle($fill);
        }
        $this->writer->endElement();
        if (!empty($hAlign)) {
            $hAlign = $this->mapHorizontalAlignment($hAlign);
            $this->writer->startElement('style:paragraph-properties');
            $this->writer->writeAttribute('fo:text-align', $hAlign);
            $this->writer->endElement();
        }
    }
    protected function mapUnderlineStyle(\ScopedGtbabel\PhpOffice\PhpSpreadsheet\Style\Font $font) : string
    {
        switch ($font->getUnderline()) {
            case \ScopedGtbabel\PhpOffice\PhpSpreadsheet\Style\Font::UNDERLINE_DOUBLE:
            case \ScopedGtbabel\PhpOffice\PhpSpreadsheet\Style\Font::UNDERLINE_DOUBLEACCOUNTING:
                return 'double';
            case \ScopedGtbabel\PhpOffice\PhpSpreadsheet\Style\Font::UNDERLINE_SINGLE:
            case \ScopedGtbabel\PhpOffice\PhpSpreadsheet\Style\Font::UNDERLINE_SINGLEACCOUNTING:
                return 'single';
        }
        return 'none';
    }
    protected function writeTextProperties(\ScopedGtbabel\PhpOffice\PhpSpreadsheet\Style\Style $style) : void
    {
        // Font
        $this->writer->startElement('style:text-properties');
        $font = $style->getFont();
        if ($font->getBold()) {
            $this->writer->writeAttribute('fo:font-weight', 'bold');
            $this->writer->writeAttribute('style:font-weight-complex', 'bold');
            $this->writer->writeAttribute('style:font-weight-asian', 'bold');
        }
        if ($font->getItalic()) {
            $this->writer->writeAttribute('fo:font-style', 'italic');
        }
        if ($color = $font->getColor()) {
            $this->writer->writeAttribute('fo:color', \sprintf('#%s', $color->getRGB()));
        }
        if ($family = $font->getName()) {
            $this->writer->writeAttribute('fo:font-family', $family);
        }
        if ($size = $font->getSize()) {
            $this->writer->writeAttribute('fo:font-size', \sprintf('%.1Fpt', $size));
        }
        if ($font->getUnderline() && $font->getUnderline() !== \ScopedGtbabel\PhpOffice\PhpSpreadsheet\Style\Font::UNDERLINE_NONE) {
            $this->writer->writeAttribute('style:text-underline-style', 'solid');
            $this->writer->writeAttribute('style:text-underline-width', 'auto');
            $this->writer->writeAttribute('style:text-underline-color', 'font-color');
            $underline = $this->mapUnderlineStyle($font);
            $this->writer->writeAttribute('style:text-underline-type', $underline);
        }
        $this->writer->endElement();
        // Close style:text-properties
    }
    public function write(\ScopedGtbabel\PhpOffice\PhpSpreadsheet\Style\Style $style) : void
    {
        $this->writer->startElement('style:style');
        $this->writer->writeAttribute('style:name', self::CELL_STYLE_PREFIX . $style->getIndex());
        $this->writer->writeAttribute('style:family', 'table-cell');
        $this->writer->writeAttribute('style:parent-style-name', 'Default');
        // Alignment, fill colour, etc
        $this->writeCellProperties($style);
        // style:text-properties
        $this->writeTextProperties($style);
        // End
        $this->writer->endElement();
        // Close style:style
    }
}
