<?php

namespace ScopedGtbabel\PhpOffice\PhpSpreadsheet\Writer;

use ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Calculation;
use ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Functions;
use ScopedGtbabel\PhpOffice\PhpSpreadsheet\HashTable;
use ScopedGtbabel\PhpOffice\PhpSpreadsheet\Spreadsheet;
use ScopedGtbabel\PhpOffice\PhpSpreadsheet\Style\Borders;
use ScopedGtbabel\PhpOffice\PhpSpreadsheet\Style\Conditional;
use ScopedGtbabel\PhpOffice\PhpSpreadsheet\Style\Fill;
use ScopedGtbabel\PhpOffice\PhpSpreadsheet\Style\Font;
use ScopedGtbabel\PhpOffice\PhpSpreadsheet\Style\NumberFormat;
use ScopedGtbabel\PhpOffice\PhpSpreadsheet\Worksheet\BaseDrawing;
use ScopedGtbabel\PhpOffice\PhpSpreadsheet\Worksheet\Drawing as WorksheetDrawing;
use ScopedGtbabel\PhpOffice\PhpSpreadsheet\Worksheet\MemoryDrawing;
use ScopedGtbabel\PhpOffice\PhpSpreadsheet\Writer\Exception as WriterException;
use ScopedGtbabel\PhpOffice\PhpSpreadsheet\Writer\Xlsx\Chart;
use ScopedGtbabel\PhpOffice\PhpSpreadsheet\Writer\Xlsx\Comments;
use ScopedGtbabel\PhpOffice\PhpSpreadsheet\Writer\Xlsx\ContentTypes;
use ScopedGtbabel\PhpOffice\PhpSpreadsheet\Writer\Xlsx\DocProps;
use ScopedGtbabel\PhpOffice\PhpSpreadsheet\Writer\Xlsx\Drawing;
use ScopedGtbabel\PhpOffice\PhpSpreadsheet\Writer\Xlsx\Rels;
use ScopedGtbabel\PhpOffice\PhpSpreadsheet\Writer\Xlsx\RelsRibbon;
use ScopedGtbabel\PhpOffice\PhpSpreadsheet\Writer\Xlsx\RelsVBA;
use ScopedGtbabel\PhpOffice\PhpSpreadsheet\Writer\Xlsx\StringTable;
use ScopedGtbabel\PhpOffice\PhpSpreadsheet\Writer\Xlsx\Style;
use ScopedGtbabel\PhpOffice\PhpSpreadsheet\Writer\Xlsx\Theme;
use ScopedGtbabel\PhpOffice\PhpSpreadsheet\Writer\Xlsx\Workbook;
use ScopedGtbabel\PhpOffice\PhpSpreadsheet\Writer\Xlsx\Worksheet;
use ZipArchive;
use ScopedGtbabel\ZipStream\Exception\OverflowException;
use ScopedGtbabel\ZipStream\Option\Archive;
use ScopedGtbabel\ZipStream\ZipStream;
class Xlsx extends \ScopedGtbabel\PhpOffice\PhpSpreadsheet\Writer\BaseWriter
{
    /**
     * Office2003 compatibility.
     *
     * @var bool
     */
    private $office2003compatibility = \false;
    /**
     * Private Spreadsheet.
     *
     * @var Spreadsheet
     */
    private $spreadSheet;
    /**
     * Private string table.
     *
     * @var string[]
     */
    private $stringTable = [];
    /**
     * Private unique Conditional HashTable.
     *
     * @var HashTable<Conditional>
     */
    private $stylesConditionalHashTable;
    /**
     * Private unique Style HashTable.
     *
     * @var HashTable<\PhpOffice\PhpSpreadsheet\Style\Style>
     */
    private $styleHashTable;
    /**
     * Private unique Fill HashTable.
     *
     * @var HashTable<Fill>
     */
    private $fillHashTable;
    /**
     * Private unique \PhpOffice\PhpSpreadsheet\Style\Font HashTable.
     *
     * @var HashTable<Font>
     */
    private $fontHashTable;
    /**
     * Private unique Borders HashTable.
     *
     * @var HashTable<Borders>
     */
    private $bordersHashTable;
    /**
     * Private unique NumberFormat HashTable.
     *
     * @var HashTable<NumberFormat>
     */
    private $numFmtHashTable;
    /**
     * Private unique \PhpOffice\PhpSpreadsheet\Worksheet\Worksheet\BaseDrawing HashTable.
     *
     * @var HashTable<BaseDrawing>
     */
    private $drawingHashTable;
    /**
     * Private handle for zip stream.
     *
     * @var ZipStream
     */
    private $zip;
    /**
     * @var Chart
     */
    private $writerPartChart;
    /**
     * @var Comments
     */
    private $writerPartComments;
    /**
     * @var ContentTypes
     */
    private $writerPartContentTypes;
    /**
     * @var DocProps
     */
    private $writerPartDocProps;
    /**
     * @var Drawing
     */
    private $writerPartDrawing;
    /**
     * @var Rels
     */
    private $writerPartRels;
    /**
     * @var RelsRibbon
     */
    private $writerPartRelsRibbon;
    /**
     * @var RelsVBA
     */
    private $writerPartRelsVBA;
    /**
     * @var StringTable
     */
    private $writerPartStringTable;
    /**
     * @var Style
     */
    private $writerPartStyle;
    /**
     * @var Theme
     */
    private $writerPartTheme;
    /**
     * @var Workbook
     */
    private $writerPartWorkbook;
    /**
     * @var Worksheet
     */
    private $writerPartWorksheet;
    /**
     * Create a new Xlsx Writer.
     */
    public function __construct(\ScopedGtbabel\PhpOffice\PhpSpreadsheet\Spreadsheet $spreadsheet)
    {
        // Assign PhpSpreadsheet
        $this->setSpreadsheet($spreadsheet);
        $this->writerPartChart = new \ScopedGtbabel\PhpOffice\PhpSpreadsheet\Writer\Xlsx\Chart($this);
        $this->writerPartComments = new \ScopedGtbabel\PhpOffice\PhpSpreadsheet\Writer\Xlsx\Comments($this);
        $this->writerPartContentTypes = new \ScopedGtbabel\PhpOffice\PhpSpreadsheet\Writer\Xlsx\ContentTypes($this);
        $this->writerPartDocProps = new \ScopedGtbabel\PhpOffice\PhpSpreadsheet\Writer\Xlsx\DocProps($this);
        $this->writerPartDrawing = new \ScopedGtbabel\PhpOffice\PhpSpreadsheet\Writer\Xlsx\Drawing($this);
        $this->writerPartRels = new \ScopedGtbabel\PhpOffice\PhpSpreadsheet\Writer\Xlsx\Rels($this);
        $this->writerPartRelsRibbon = new \ScopedGtbabel\PhpOffice\PhpSpreadsheet\Writer\Xlsx\RelsRibbon($this);
        $this->writerPartRelsVBA = new \ScopedGtbabel\PhpOffice\PhpSpreadsheet\Writer\Xlsx\RelsVBA($this);
        $this->writerPartStringTable = new \ScopedGtbabel\PhpOffice\PhpSpreadsheet\Writer\Xlsx\StringTable($this);
        $this->writerPartStyle = new \ScopedGtbabel\PhpOffice\PhpSpreadsheet\Writer\Xlsx\Style($this);
        $this->writerPartTheme = new \ScopedGtbabel\PhpOffice\PhpSpreadsheet\Writer\Xlsx\Theme($this);
        $this->writerPartWorkbook = new \ScopedGtbabel\PhpOffice\PhpSpreadsheet\Writer\Xlsx\Workbook($this);
        $this->writerPartWorksheet = new \ScopedGtbabel\PhpOffice\PhpSpreadsheet\Writer\Xlsx\Worksheet($this);
        // Set HashTable variables
        // @phpstan-ignore-next-line
        $this->bordersHashTable = new \ScopedGtbabel\PhpOffice\PhpSpreadsheet\HashTable();
        // @phpstan-ignore-next-line
        $this->drawingHashTable = new \ScopedGtbabel\PhpOffice\PhpSpreadsheet\HashTable();
        // @phpstan-ignore-next-line
        $this->fillHashTable = new \ScopedGtbabel\PhpOffice\PhpSpreadsheet\HashTable();
        // @phpstan-ignore-next-line
        $this->fontHashTable = new \ScopedGtbabel\PhpOffice\PhpSpreadsheet\HashTable();
        // @phpstan-ignore-next-line
        $this->numFmtHashTable = new \ScopedGtbabel\PhpOffice\PhpSpreadsheet\HashTable();
        // @phpstan-ignore-next-line
        $this->styleHashTable = new \ScopedGtbabel\PhpOffice\PhpSpreadsheet\HashTable();
        // @phpstan-ignore-next-line
        $this->stylesConditionalHashTable = new \ScopedGtbabel\PhpOffice\PhpSpreadsheet\HashTable();
    }
    public function getWriterPartChart() : \ScopedGtbabel\PhpOffice\PhpSpreadsheet\Writer\Xlsx\Chart
    {
        return $this->writerPartChart;
    }
    public function getWriterPartComments() : \ScopedGtbabel\PhpOffice\PhpSpreadsheet\Writer\Xlsx\Comments
    {
        return $this->writerPartComments;
    }
    public function getWriterPartContentTypes() : \ScopedGtbabel\PhpOffice\PhpSpreadsheet\Writer\Xlsx\ContentTypes
    {
        return $this->writerPartContentTypes;
    }
    public function getWriterPartDocProps() : \ScopedGtbabel\PhpOffice\PhpSpreadsheet\Writer\Xlsx\DocProps
    {
        return $this->writerPartDocProps;
    }
    public function getWriterPartDrawing() : \ScopedGtbabel\PhpOffice\PhpSpreadsheet\Writer\Xlsx\Drawing
    {
        return $this->writerPartDrawing;
    }
    public function getWriterPartRels() : \ScopedGtbabel\PhpOffice\PhpSpreadsheet\Writer\Xlsx\Rels
    {
        return $this->writerPartRels;
    }
    public function getWriterPartRelsRibbon() : \ScopedGtbabel\PhpOffice\PhpSpreadsheet\Writer\Xlsx\RelsRibbon
    {
        return $this->writerPartRelsRibbon;
    }
    public function getWriterPartRelsVBA() : \ScopedGtbabel\PhpOffice\PhpSpreadsheet\Writer\Xlsx\RelsVBA
    {
        return $this->writerPartRelsVBA;
    }
    public function getWriterPartStringTable() : \ScopedGtbabel\PhpOffice\PhpSpreadsheet\Writer\Xlsx\StringTable
    {
        return $this->writerPartStringTable;
    }
    public function getWriterPartStyle() : \ScopedGtbabel\PhpOffice\PhpSpreadsheet\Writer\Xlsx\Style
    {
        return $this->writerPartStyle;
    }
    public function getWriterPartTheme() : \ScopedGtbabel\PhpOffice\PhpSpreadsheet\Writer\Xlsx\Theme
    {
        return $this->writerPartTheme;
    }
    public function getWriterPartWorkbook() : \ScopedGtbabel\PhpOffice\PhpSpreadsheet\Writer\Xlsx\Workbook
    {
        return $this->writerPartWorkbook;
    }
    public function getWriterPartWorksheet() : \ScopedGtbabel\PhpOffice\PhpSpreadsheet\Writer\Xlsx\Worksheet
    {
        return $this->writerPartWorksheet;
    }
    /**
     * Save PhpSpreadsheet to file.
     *
     * @param resource|string $filename
     */
    public function save($filename, int $flags = 0) : void
    {
        $this->processFlags($flags);
        // garbage collect
        $this->pathNames = [];
        $this->spreadSheet->garbageCollect();
        $saveDebugLog = \ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Calculation::getInstance($this->spreadSheet)->getDebugLog()->getWriteDebugLog();
        \ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Calculation::getInstance($this->spreadSheet)->getDebugLog()->setWriteDebugLog(\false);
        $saveDateReturnType = \ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Functions::getReturnDateType();
        \ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Functions::setReturnDateType(\ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Functions::RETURNDATE_EXCEL);
        // Create string lookup table
        $this->stringTable = [];
        for ($i = 0; $i < $this->spreadSheet->getSheetCount(); ++$i) {
            $this->stringTable = $this->getWriterPartStringTable()->createStringTable($this->spreadSheet->getSheet($i), $this->stringTable);
        }
        // Create styles dictionaries
        $this->styleHashTable->addFromSource($this->getWriterPartStyle()->allStyles($this->spreadSheet));
        $this->stylesConditionalHashTable->addFromSource($this->getWriterPartStyle()->allConditionalStyles($this->spreadSheet));
        $this->fillHashTable->addFromSource($this->getWriterPartStyle()->allFills($this->spreadSheet));
        $this->fontHashTable->addFromSource($this->getWriterPartStyle()->allFonts($this->spreadSheet));
        $this->bordersHashTable->addFromSource($this->getWriterPartStyle()->allBorders($this->spreadSheet));
        $this->numFmtHashTable->addFromSource($this->getWriterPartStyle()->allNumberFormats($this->spreadSheet));
        // Create drawing dictionary
        $this->drawingHashTable->addFromSource($this->getWriterPartDrawing()->allDrawings($this->spreadSheet));
        $zipContent = [];
        // Add [Content_Types].xml to ZIP file
        $zipContent['[Content_Types].xml'] = $this->getWriterPartContentTypes()->writeContentTypes($this->spreadSheet, $this->includeCharts);
        //if hasMacros, add the vbaProject.bin file, Certificate file(if exists)
        if ($this->spreadSheet->hasMacros()) {
            $macrosCode = $this->spreadSheet->getMacrosCode();
            if ($macrosCode !== null) {
                // we have the code ?
                $zipContent['xl/vbaProject.bin'] = $macrosCode;
                //allways in 'xl', allways named vbaProject.bin
                if ($this->spreadSheet->hasMacrosCertificate()) {
                    //signed macros ?
                    // Yes : add the certificate file and the related rels file
                    $zipContent['xl/vbaProjectSignature.bin'] = $this->spreadSheet->getMacrosCertificate();
                    $zipContent['xl/_rels/vbaProject.bin.rels'] = $this->getWriterPartRelsVBA()->writeVBARelationships($this->spreadSheet);
                }
            }
        }
        //a custom UI in this workbook ? add it ("base" xml and additional objects (pictures) and rels)
        if ($this->spreadSheet->hasRibbon()) {
            $tmpRibbonTarget = $this->spreadSheet->getRibbonXMLData('target');
            $zipContent[$tmpRibbonTarget] = $this->spreadSheet->getRibbonXMLData('data');
            if ($this->spreadSheet->hasRibbonBinObjects()) {
                $tmpRootPath = \dirname($tmpRibbonTarget) . '/';
                $ribbonBinObjects = $this->spreadSheet->getRibbonBinObjects('data');
                //the files to write
                foreach ($ribbonBinObjects as $aPath => $aContent) {
                    $zipContent[$tmpRootPath . $aPath] = $aContent;
                }
                //the rels for files
                $zipContent[$tmpRootPath . '_rels/' . \basename($tmpRibbonTarget) . '.rels'] = $this->getWriterPartRelsRibbon()->writeRibbonRelationships($this->spreadSheet);
            }
        }
        // Add relationships to ZIP file
        $zipContent['_rels/.rels'] = $this->getWriterPartRels()->writeRelationships($this->spreadSheet);
        $zipContent['xl/_rels/workbook.xml.rels'] = $this->getWriterPartRels()->writeWorkbookRelationships($this->spreadSheet);
        // Add document properties to ZIP file
        $zipContent['docProps/app.xml'] = $this->getWriterPartDocProps()->writeDocPropsApp($this->spreadSheet);
        $zipContent['docProps/core.xml'] = $this->getWriterPartDocProps()->writeDocPropsCore($this->spreadSheet);
        $customPropertiesPart = $this->getWriterPartDocProps()->writeDocPropsCustom($this->spreadSheet);
        if ($customPropertiesPart !== null) {
            $zipContent['docProps/custom.xml'] = $customPropertiesPart;
        }
        // Add theme to ZIP file
        $zipContent['xl/theme/theme1.xml'] = $this->getWriterPartTheme()->writeTheme($this->spreadSheet);
        // Add string table to ZIP file
        $zipContent['xl/sharedStrings.xml'] = $this->getWriterPartStringTable()->writeStringTable($this->stringTable);
        // Add styles to ZIP file
        $zipContent['xl/styles.xml'] = $this->getWriterPartStyle()->writeStyles($this->spreadSheet);
        // Add workbook to ZIP file
        $zipContent['xl/workbook.xml'] = $this->getWriterPartWorkbook()->writeWorkbook($this->spreadSheet, $this->preCalculateFormulas);
        $chartCount = 0;
        // Add worksheets
        for ($i = 0; $i < $this->spreadSheet->getSheetCount(); ++$i) {
            $zipContent['xl/worksheets/sheet' . ($i + 1) . '.xml'] = $this->getWriterPartWorksheet()->writeWorksheet($this->spreadSheet->getSheet($i), $this->stringTable, $this->includeCharts);
            if ($this->includeCharts) {
                $charts = $this->spreadSheet->getSheet($i)->getChartCollection();
                if (\count($charts) > 0) {
                    foreach ($charts as $chart) {
                        $zipContent['xl/charts/chart' . ($chartCount + 1) . '.xml'] = $this->getWriterPartChart()->writeChart($chart, $this->preCalculateFormulas);
                        ++$chartCount;
                    }
                }
            }
        }
        $chartRef1 = 0;
        // Add worksheet relationships (drawings, ...)
        for ($i = 0; $i < $this->spreadSheet->getSheetCount(); ++$i) {
            // Add relationships
            $zipContent['xl/worksheets/_rels/sheet' . ($i + 1) . '.xml.rels'] = $this->getWriterPartRels()->writeWorksheetRelationships($this->spreadSheet->getSheet($i), $i + 1, $this->includeCharts);
            // Add unparsedLoadedData
            $sheetCodeName = $this->spreadSheet->getSheet($i)->getCodeName();
            $unparsedLoadedData = $this->spreadSheet->getUnparsedLoadedData();
            if (isset($unparsedLoadedData['sheets'][$sheetCodeName]['ctrlProps'])) {
                foreach ($unparsedLoadedData['sheets'][$sheetCodeName]['ctrlProps'] as $ctrlProp) {
                    $zipContent[$ctrlProp['filePath']] = $ctrlProp['content'];
                }
            }
            if (isset($unparsedLoadedData['sheets'][$sheetCodeName]['printerSettings'])) {
                foreach ($unparsedLoadedData['sheets'][$sheetCodeName]['printerSettings'] as $ctrlProp) {
                    $zipContent[$ctrlProp['filePath']] = $ctrlProp['content'];
                }
            }
            $drawings = $this->spreadSheet->getSheet($i)->getDrawingCollection();
            $drawingCount = \count($drawings);
            if ($this->includeCharts) {
                $chartCount = $this->spreadSheet->getSheet($i)->getChartCount();
            }
            // Add drawing and image relationship parts
            if ($drawingCount > 0 || $chartCount > 0) {
                // Drawing relationships
                $zipContent['xl/drawings/_rels/drawing' . ($i + 1) . '.xml.rels'] = $this->getWriterPartRels()->writeDrawingRelationships($this->spreadSheet->getSheet($i), $chartRef1, $this->includeCharts);
                // Drawings
                $zipContent['xl/drawings/drawing' . ($i + 1) . '.xml'] = $this->getWriterPartDrawing()->writeDrawings($this->spreadSheet->getSheet($i), $this->includeCharts);
            } elseif (isset($unparsedLoadedData['sheets'][$sheetCodeName]['drawingAlternateContents'])) {
                // Drawings
                $zipContent['xl/drawings/drawing' . ($i + 1) . '.xml'] = $this->getWriterPartDrawing()->writeDrawings($this->spreadSheet->getSheet($i), $this->includeCharts);
            }
            // Add unparsed drawings
            if (isset($unparsedLoadedData['sheets'][$sheetCodeName]['Drawings'])) {
                foreach ($unparsedLoadedData['sheets'][$sheetCodeName]['Drawings'] as $relId => $drawingXml) {
                    $drawingFile = \array_search($relId, $unparsedLoadedData['sheets'][$sheetCodeName]['drawingOriginalIds']);
                    if ($drawingFile !== \false) {
                        $drawingFile = \ltrim($drawingFile, '.');
                        $zipContent['xl' . $drawingFile] = $drawingXml;
                    }
                }
            }
            // Add comment relationship parts
            if (\count($this->spreadSheet->getSheet($i)->getComments()) > 0) {
                // VML Comments
                $zipContent['xl/drawings/vmlDrawing' . ($i + 1) . '.vml'] = $this->getWriterPartComments()->writeVMLComments($this->spreadSheet->getSheet($i));
                // Comments
                $zipContent['xl/comments' . ($i + 1) . '.xml'] = $this->getWriterPartComments()->writeComments($this->spreadSheet->getSheet($i));
            }
            // Add unparsed relationship parts
            if (isset($unparsedLoadedData['sheets'][$sheetCodeName]['vmlDrawings'])) {
                foreach ($unparsedLoadedData['sheets'][$sheetCodeName]['vmlDrawings'] as $vmlDrawing) {
                    $zipContent[$vmlDrawing['filePath']] = $vmlDrawing['content'];
                }
            }
            // Add header/footer relationship parts
            if (\count($this->spreadSheet->getSheet($i)->getHeaderFooter()->getImages()) > 0) {
                // VML Drawings
                $zipContent['xl/drawings/vmlDrawingHF' . ($i + 1) . '.vml'] = $this->getWriterPartDrawing()->writeVMLHeaderFooterImages($this->spreadSheet->getSheet($i));
                // VML Drawing relationships
                $zipContent['xl/drawings/_rels/vmlDrawingHF' . ($i + 1) . '.vml.rels'] = $this->getWriterPartRels()->writeHeaderFooterDrawingRelationships($this->spreadSheet->getSheet($i));
                // Media
                foreach ($this->spreadSheet->getSheet($i)->getHeaderFooter()->getImages() as $image) {
                    $zipContent['xl/media/' . $image->getIndexedFilename()] = \file_get_contents($image->getPath());
                }
            }
        }
        // Add media
        for ($i = 0; $i < $this->getDrawingHashTable()->count(); ++$i) {
            if ($this->getDrawingHashTable()->getByIndex($i) instanceof \ScopedGtbabel\PhpOffice\PhpSpreadsheet\Worksheet\Drawing) {
                $imageContents = null;
                $imagePath = $this->getDrawingHashTable()->getByIndex($i)->getPath();
                if (\strpos($imagePath, 'zip://') !== \false) {
                    $imagePath = \substr($imagePath, 6);
                    $imagePathSplitted = \explode('#', $imagePath);
                    $imageZip = new \ZipArchive();
                    $imageZip->open($imagePathSplitted[0]);
                    $imageContents = $imageZip->getFromName($imagePathSplitted[1]);
                    $imageZip->close();
                    unset($imageZip);
                } else {
                    $imageContents = \file_get_contents($imagePath);
                }
                $zipContent['xl/media/' . \str_replace(' ', '_', $this->getDrawingHashTable()->getByIndex($i)->getIndexedFilename())] = $imageContents;
            } elseif ($this->getDrawingHashTable()->getByIndex($i) instanceof \ScopedGtbabel\PhpOffice\PhpSpreadsheet\Worksheet\MemoryDrawing) {
                \ob_start();
                \call_user_func($this->getDrawingHashTable()->getByIndex($i)->getRenderingFunction(), $this->getDrawingHashTable()->getByIndex($i)->getImageResource());
                $imageContents = \ob_get_contents();
                \ob_end_clean();
                $zipContent['xl/media/' . \str_replace(' ', '_', $this->getDrawingHashTable()->getByIndex($i)->getIndexedFilename())] = $imageContents;
            }
        }
        \ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Functions::setReturnDateType($saveDateReturnType);
        \ScopedGtbabel\PhpOffice\PhpSpreadsheet\Calculation\Calculation::getInstance($this->spreadSheet)->getDebugLog()->setWriteDebugLog($saveDebugLog);
        $this->openFileHandle($filename);
        $options = new \ScopedGtbabel\ZipStream\Option\Archive();
        $options->setEnableZip64(\false);
        $options->setOutputStream($this->fileHandle);
        $this->zip = new \ScopedGtbabel\ZipStream\ZipStream(null, $options);
        $this->addZipFiles($zipContent);
        // Close file
        try {
            $this->zip->finish();
        } catch (\ScopedGtbabel\ZipStream\Exception\OverflowException $e) {
            throw new \ScopedGtbabel\PhpOffice\PhpSpreadsheet\Writer\Exception('Could not close resource.');
        }
        $this->maybeCloseFileHandle();
    }
    /**
     * Get Spreadsheet object.
     *
     * @return Spreadsheet
     */
    public function getSpreadsheet()
    {
        return $this->spreadSheet;
    }
    /**
     * Set Spreadsheet object.
     *
     * @param Spreadsheet $spreadsheet PhpSpreadsheet object
     *
     * @return $this
     */
    public function setSpreadsheet(\ScopedGtbabel\PhpOffice\PhpSpreadsheet\Spreadsheet $spreadsheet)
    {
        $this->spreadSheet = $spreadsheet;
        return $this;
    }
    /**
     * Get string table.
     *
     * @return string[]
     */
    public function getStringTable()
    {
        return $this->stringTable;
    }
    /**
     * Get Style HashTable.
     *
     * @return HashTable<\PhpOffice\PhpSpreadsheet\Style\Style>
     */
    public function getStyleHashTable()
    {
        return $this->styleHashTable;
    }
    /**
     * Get Conditional HashTable.
     *
     * @return HashTable<Conditional>
     */
    public function getStylesConditionalHashTable()
    {
        return $this->stylesConditionalHashTable;
    }
    /**
     * Get Fill HashTable.
     *
     * @return HashTable<Fill>
     */
    public function getFillHashTable()
    {
        return $this->fillHashTable;
    }
    /**
     * Get \PhpOffice\PhpSpreadsheet\Style\Font HashTable.
     *
     * @return HashTable<Font>
     */
    public function getFontHashTable()
    {
        return $this->fontHashTable;
    }
    /**
     * Get Borders HashTable.
     *
     * @return HashTable<Borders>
     */
    public function getBordersHashTable()
    {
        return $this->bordersHashTable;
    }
    /**
     * Get NumberFormat HashTable.
     *
     * @return HashTable<NumberFormat>
     */
    public function getNumFmtHashTable()
    {
        return $this->numFmtHashTable;
    }
    /**
     * Get \PhpOffice\PhpSpreadsheet\Worksheet\Worksheet\BaseDrawing HashTable.
     *
     * @return HashTable<BaseDrawing>
     */
    public function getDrawingHashTable()
    {
        return $this->drawingHashTable;
    }
    /**
     * Get Office2003 compatibility.
     *
     * @return bool
     */
    public function getOffice2003Compatibility()
    {
        return $this->office2003compatibility;
    }
    /**
     * Set Office2003 compatibility.
     *
     * @param bool $pValue Office2003 compatibility?
     *
     * @return $this
     */
    public function setOffice2003Compatibility($pValue)
    {
        $this->office2003compatibility = $pValue;
        return $this;
    }
    private $pathNames = [];
    private function addZipFile(string $path, string $content) : void
    {
        if (!\in_array($path, $this->pathNames)) {
            $this->pathNames[] = $path;
            $this->zip->addFile($path, $content);
        }
    }
    private function addZipFiles(array $zipContent) : void
    {
        foreach ($zipContent as $path => $content) {
            $this->addZipFile($path, $content);
        }
    }
}
