<?php

declare (strict_types=1);
namespace ScopedGtbabel\Dotenv\Loader;

use ScopedGtbabel\Dotenv\Parser\Entry;
use ScopedGtbabel\Dotenv\Parser\Value;
use ScopedGtbabel\Dotenv\Repository\RepositoryInterface;
final class Loader implements \ScopedGtbabel\Dotenv\Loader\LoaderInterface
{
    /**
     * Load the given entries into the repository.
     *
     * We'll substitute any nested variables, and send each variable to the
     * repository, with the effect of actually mutating the environment.
     *
     * @param \Dotenv\Repository\RepositoryInterface $repository
     * @param \Dotenv\Parser\Entry[]                 $entries
     *
     * @return array<string,string|null>
     */
    public function load(\ScopedGtbabel\Dotenv\Repository\RepositoryInterface $repository, array $entries)
    {
        return \array_reduce($entries, static function (array $vars, \ScopedGtbabel\Dotenv\Parser\Entry $entry) use($repository) {
            $name = $entry->getName();
            $value = $entry->getValue()->map(static function (\ScopedGtbabel\Dotenv\Parser\Value $value) use($repository) {
                return \ScopedGtbabel\Dotenv\Loader\Resolver::resolve($repository, $value);
            });
            if ($value->isDefined()) {
                $inner = $value->get();
                if ($repository->set($name, $inner)) {
                    return \array_merge($vars, [$name => $inner]);
                }
            } else {
                if ($repository->clear($name)) {
                    return \array_merge($vars, [$name => null]);
                }
            }
            return $vars;
        }, []);
    }
}
