<?php

declare (strict_types=1);
namespace ScopedGtbabel\Dotenv\Loader;

use ScopedGtbabel\Dotenv\Parser\Value;
use ScopedGtbabel\Dotenv\Repository\RepositoryInterface;
use ScopedGtbabel\Dotenv\Util\Regex;
use ScopedGtbabel\Dotenv\Util\Str;
use ScopedGtbabel\PhpOption\Option;
final class Resolver
{
    /**
     * This class is a singleton.
     *
     * @codeCoverageIgnore
     *
     * @return void
     */
    private function __construct()
    {
        //
    }
    /**
     * Resolve the nested variables in the given value.
     *
     * Replaces ${varname} patterns in the allowed positions in the variable
     * value by an existing environment variable.
     *
     * @param \Dotenv\Repository\RepositoryInterface $repository
     * @param \Dotenv\Parser\Value                   $value
     *
     * @return string
     */
    public static function resolve(\ScopedGtbabel\Dotenv\Repository\RepositoryInterface $repository, \ScopedGtbabel\Dotenv\Parser\Value $value)
    {
        return \array_reduce($value->getVars(), static function (string $s, int $i) use($repository) {
            return \ScopedGtbabel\Dotenv\Util\Str::substr($s, 0, $i) . self::resolveVariable($repository, \ScopedGtbabel\Dotenv\Util\Str::substr($s, $i));
        }, $value->getChars());
    }
    /**
     * Resolve a single nested variable.
     *
     * @param \Dotenv\Repository\RepositoryInterface $repository
     * @param string                                 $str
     *
     * @return string
     */
    private static function resolveVariable(\ScopedGtbabel\Dotenv\Repository\RepositoryInterface $repository, string $str)
    {
        return \ScopedGtbabel\Dotenv\Util\Regex::replaceCallback('/\\A\\${([a-zA-Z0-9_.]+)}/', static function (array $matches) use($repository) {
            return \ScopedGtbabel\PhpOption\Option::fromValue($repository->get($matches[1]))->getOrElse($matches[0]);
        }, $str, 1)->success()->getOrElse($str);
    }
}
